Package: detzrcr
Authors@R: c(
    person("Magnus", "Kristoffersen", email="magnus.kristoffersen@geo.uio.no",
    role=c("aut", "cre")))
Type: Package
Title: Compare Detrital Zircon Suites
Version: 0.3.3
Description: Compare detrital zircon suites by uploading univariate,
      U-Pb age, or bivariate, U-Pb age and Lu-Hf data, in a 'shiny'-based
      user-interface. Outputs publication quality figures using 'ggplot2', and
      tables of statistics currently in use in the detrital zircon geochronology
      community.
License: GPL-3
URL: https://github.com/magnuskristoffersen/detzrcr
RoxygenNote: 7.3.3
Imports: ggplot2, shiny, MASS, stats, graphics, utils, DT, yaml
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-30 10:19:07 UTC; magnukr
Author: Magnus Kristoffersen [aut, cre]
Maintainer: Magnus Kristoffersen <magnus.kristoffersen@geo.uio.no>
Repository: CRAN
Date/Publication: 2026-05-04 06:50:02 UTC
Built: R 4.6.0; ; 2026-05-04 08:09:08 UTC; unix

Package: detzrcr
Authors@R: c(
    person("Magnus", "Kristoffersen", email="magnus.kristoffersen@geo.uio.no",
    role=c("aut", "cre")))
Type: Package
Title: Compare Detrital Zircon Suites
Version: 0.3.3
Description: Compare detrital zircon suites by uploading univariate,
      U-Pb age, or bivariate, U-Pb age and Lu-Hf data, in a 'shiny'-based
      user-interface. Outputs publication quality figures using 'ggplot2', and
      tables of statistics currently in use in the detrital zircon geochronology
      community.
License: GPL-3
URL: https://github.com/magnuskristoffersen/detzrcr
RoxygenNote: 7.3.3
Imports: ggplot2, shiny, MASS, stats, graphics, utils, DT, yaml
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-30 10:19:07 UTC; magnukr
Author: Magnus Kristoffersen [aut, cre]
Maintainer: Magnus Kristoffersen <magnus.kristoffersen@geo.uio.no>
Repository: CRAN
Date/Publication: 2026-05-04 06:50:02 UTC
Built: R 4.6.0; ; 2026-05-05 00:24:29 UTC; unix

# Generated by roxygen2: do not edit by hand

export(calc_ab)
export(calc_dens)
export(calc_dens_hist)
export(calc_dkw)
export(calc_hf)
export(calc_o_param)
export(calc_p_apply)
export(calc_quantiles)
export(check_conc)
export(combine_matrices)
export(concX)
export(concY)
export(dzr_mix)
export(find_maxima)
export(find_plot_max)
export(find_plot_min)
export(find_plot_min_max)
export(hf_lines)
export(make_tiling)
export(o_param_matrix_age)
export(o_param_matrix_tdm)
export(plot_axis_lim)
export(plot_bw_theme)
export(plot_dens)
export(plot_dens_hist)
export(plot_ecdf)
export(plot_hf)
export(plot_labels)
export(plot_point_scale)
export(plot_quantiles)
export(plot_reimink)
export(plot_text_options)
export(plot_tile)
export(populate_matrix)
export(quant_bounds)
export(reimink)
export(run_detzrcr)
export(satkoski_1d)
export(satkoski_1d_matrix)
export(satkoski_2d)
export(satkoski_2d_matrix)
export(tile_func)
export(tiling)

# detzrcr

<!-- badges: start -->
[![R-CMD-check](https://github.com/magnuskristoffersen/detzrcr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/magnuskristoffersen/detzrcr/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/detzrcr)](https://cran.r-project.org/package=detzrcr)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.3324801.svg)](https://doi.org/10.5281/zenodo.3324801)
<!-- badges: end -->

detzrcr is an R-package for plotting and running statistical tests on univariate
(U-Pb age) and bivariate (U-Pb age and Lu-Hf data) detrital zircon data for
comparison between samples. The package uses a user interface built with shiny
and produces publication quality figures using ggplot2.

## Installation

Install either from CRAN

```R
install.packages("detzrcr")
```

or install the development version from github (requires remotes)

```R
# install.packages("remotes")
remotes::install_github("magnuskristoffersen/detzrcr")
```

## Use package

Launch user interface with

```R
library("detzrcr")
run_detzrcr()
```

For the sake of simplicity input data is required to be a csv-file which
contains at least a column containing the U-Pb age data. This column must be
called age, associated uncertainty and discordance (if included) must be in
columns uncert and disc, respectively. All ages must be in Ma.

The file can contain the Lu-Hf data in two ways:
measured 176/177Hf and measured Lu/Hf in columns called hfhf and luhf,
respectively; or a column with initial epsilon-Hf with the name ehf_i.

Several samples can be included in the csv-file and which one(s) to plot can
be chosen interactively in the user interface. For this to work a column
called sample must be present, and each analysis (i.e. each row) must contain
the name of the sample it belongs to in this column.

Detrital zircon U-Pb and Lu-Hf example data from the Natal Group, KwaZulu-Natal,
South Africa (Kristoffersen et al. 2016) are included in the package.

### References

Kristoffersen, M., Andersen, T., Elburg, M.A., Watkeys, M.K., 2016.
Detrital zircon in a supercontinental setting: locally derived and
far-transported components in the Ordovician Natal Group, South Africa.
J. Geol. Soc. London. 173, 203-215. <https://dx.doi.org/10.1144/jgs2015-012>

consts <- yaml::read_yaml('constants.yml')
ui <- shiny::fluidPage(shiny::navbarPage("detzrcr",
                        shiny::tabPanel('Data Input', shiny::sidebarLayout(
                          shiny::sidebarPanel(
                            shiny::conditionalPanel(
                              condition = 'input.example_data == false',
                              shiny::fileInput('file1', 'Select CSV File',
                                               accept=c('text/csv',
                                                        'text/comma-separated-values,text/plain',
                                                        '.csv'), multiple = TRUE),
                              shiny::tags$hr(),
                              shiny::radioButtons('sep', 'Separator',
                                                  c(Comma=',',
                                                    Semicolon=';',
                                                    Tab='\t'),
                                                  ','),
                              shiny::radioButtons('quote', 'Quote',
                                                  c(None='',
                                                    'Double Quote'='"',
                                                    'Single Quote'="'"),
                                                  '"'),
                              shiny::tags$hr()),
                            shiny::checkboxInput('disc',
                                                 label = 'Remove discordant data',
                                                 value = FALSE),
                            shiny::uiOutput('show_disc_limit'),
                            shiny::tags$hr(),
                            shiny::checkboxInput('example_data',
                                                 'Display example data',
                                                 value=FALSE),
                            shiny::tags$hr(),
                            shiny::checkboxInput('sample_freq',
                                                 'Show sample frequencies',
                                                 value=FALSE)
                          ),
                          shiny::mainPanel(
                            DT::dataTableOutput('head'),
                            shiny::textOutput('nas'),
                            shiny::tableOutput('numbers')
                          )
                        )),

                        # Start of density distribution tab
                        shiny::tabPanel('Density Distribution ', shiny::sidebarLayout(
                          shiny::sidebarPanel(
                            shiny::radioButtons('type', 'Density distribution type',
                                                c(KDE='kde',
                                                  PDP='pdd')),
                            shiny::checkboxInput('hist', label = 'Histogram', value = TRUE),
                            shiny::checkboxInput('fixed_y', label= 'Fixed y-axis', value = FALSE),
                            shiny::selectInput('dens_type', 'Plot type',
                                               c('All samples in one'='dens_facet',
                                                 'Individual samples'='dens_ind',
                                                 'Combine samples'='dens_combine')),
                            shiny::uiOutput('dens_switch'),
                            shiny::numericInput('binwidth', 'Binwidth', 50),
                            shiny::numericInput('bw', "Bandwidth", 30),
                            shiny::numericInput('xstart', "X-axis start (Ma)",
                                                value=200,min=0,max=4600,step=100),
                            shiny::numericInput('xstop', "X-axis start (Ma)",
                                                value=4000,min=0,max=4600,step=100),
                            shiny::numericInput('xstep', 'X step', 200),
                            shiny::numericInput('densWidth', 'Image Width (cm)', 15),
                            shiny::numericInput('densHeight', 'Image Height (cm)', 15),
                            shiny::downloadButton('downloadDensplot', 'Save Image')),
                          shiny::mainPanel(
                            shiny::plotOutput('dens_plot')
                          )
                        )),

                        # Start of ECDF tab
                        shiny::tabPanel('ECDF', shiny::sidebarLayout(
                          shiny::sidebarPanel(
                            shiny::radioButtons('ecdf_input_type', 'Type',
                                                c('Age'='age',
                                                  'Model age'='t_dm2')),
                            shiny::selectInput('ecdf_type', 'Plot type',
                                               c('All samples in one'='same_plot',
                                                 'Individual samples'='ind_plot',
                                                 'Combine samples'='ecdf_combine_plot')),
                            shiny::uiOutput("ecdf_switch"),
                            shiny::checkboxInput('ecdf_conf', label='Confidence bands', value=FALSE),
                            shiny::numericInput('ecdf_xstart', 'X-axis start (Ma)',
                                                value=200,min=0,max=4600,step=100),
                            shiny::numericInput('ecdf_xstop', 'X-axis stop (Ma)',
                                                value=4000,min=0,max=4600,step=100),
                            shiny::numericInput('ecdf_xstep', 'X-axis step (Ma)', 200),
                            shiny::checkboxInput("ecdf_legend", label = "Show legend", value = TRUE),
                            shiny::numericInput('ecdf_width', 'Image Width (cm)', 15),
                            shiny::numericInput('ecdf_height', 'Image Height (cm)', 15),
                            shiny::downloadButton('download_ecdf_plot', 'Save Image')
                          ),
                          shiny::mainPanel(
                            shiny::plotOutput(('ecdf_plot'))
                          )
                        )),

                        # Start of UQ vs. tLQ tab
                        shiny::tabPanel('UQ vs. LQ', shiny::sidebarLayout(
                          shiny::sidebarPanel(
                            shiny::radioButtons('uqlq_type', 'UQ vs. LQ type',
                                                c('Age'='uqlq_age',
                                                  'Model age'='uqlq_tdm')),
                            shiny::uiOutput('uqlq_samples'),
                            shiny::numericInput('uqlq_xstep', 'X step', 500),
                            shiny::numericInput('uqlq_xstart', 'X-axis start (Ma)',
                                                value=200,min=0,max=4600,step=100),
                            shiny::numericInput('uqlq_xstop', 'X-axis stop (Ma)',
                                                value=4000,min=0,max=4600,step=100),
                            shiny::numericInput('uqlq_ystart', 'Y-axis start (Ma)',
                                                value=200,min=0,max=4600,step=100),
                            shiny::numericInput('uqlq_ystop', 'Y-axis stop (Ma)',
                                                value=4000,min=0,max=4600,step=100),
                            shiny::checkboxInput('uqlq_conf',
                                                 label='Confidence limits',
                                                 value=FALSE),
                            shiny::checkboxInput('mixing_model',
                                                 label='Add mixing model',
                                                 value=FALSE),
                            shiny::conditionalPanel(
                              condition="input.mixing_model == true",
                              shiny::numericInput('mu1', 'First mean', value=500),
                              shiny::numericInput('sig1', 'First standard deviation',
                                                  value=50),
                              shiny::numericInput('mu2', 'Second mean', value=1000),
                              shiny::numericInput('sig2', 'Second standard deviation',
                                                  value=100)),
                            shiny::tags$hr(),
                            shiny::checkboxInput("uqlq_legend", label = "Show legend", value = TRUE),
                            shiny::numericInput('uqlq_width', 'Image Width (cm)', 15),
                            shiny::numericInput('uqlq_height', 'Image Height (cm)', 15),
                            shiny::downloadButton('download_uqlq_plot', 'Save Image'),
                            shiny::tags$hr()
                          ),
                          shiny::mainPanel(
                            shiny::plotOutput(('uqlq'))
                          )
                        )),

                        # Start of Hf tab
                        shiny::tabPanel('Lu-Hf', shiny::sidebarLayout(
                          shiny::sidebarPanel(
                            shiny::radioButtons('hf_type', 'Type',
                                                c('Epsilon Hf'='ehf_plot',
                                                  'Hf/Hf'='hfhf_plot')),
                            shiny::uiOutput('hf_samples'),
                            shiny::numericInput('hf_xstart', 'X-axis start (Ma)',
                                                value=200,min=0,max=4600,step=100),
                            shiny::numericInput('hf_xstop', 'X-axis stop (Ma)',
                                                value=4000,min=0,max=4600,step=100),
                            shiny::numericInput('hf_xstep', 'X step', 200),
                            shiny::uiOutput("hf_switch"),
                            shiny::tags$hr(),
                            shiny::checkboxInput('error_bars',
                                                 label = 'Add error bars',
                                                 value = FALSE),
                            shiny::conditionalPanel(
                              condition = 'input.error_bars == true',
                              shiny::checkboxInput('x_error_bars',
                                                   label = 'X-direction error bars',
                                                   value = TRUE),
                              shiny::checkboxInput('y_error_bars',
                                                   label = 'Y-direction error bars',
                                                   value = TRUE)
                            ),
                            shiny::tags$hr(),
                            shiny::checkboxInput('add_contours',
                                                 label = 'Add contours',
                                                 value = FALSE),
                            shiny::uiOutput('hf_bandwidths'),
                            shiny::uiOutput('contour_switch'),
                            shiny::tags$hr(),
                            shiny::checkboxInput("hf_legend", label = "Show legend", value = TRUE),
                            shiny::numericInput('hf_width', 'Image Width (cm)', 15),
                            shiny::numericInput('hf_height', 'Image Height (cm)', 15),
                            shiny::downloadButton('download_hf_plot', 'Save Image'),
                            shiny::tags$hr(),
                            shiny::downloadButton('download_hf_table', 'Save Lu-Hf Table')
                          ),
                          shiny::mainPanel(
                            shiny::plotOutput(('hf'))
                          )
                        )),

                        # Start of likeness tab
                        shiny::tabPanel('Likeness', shiny::sidebarLayout(
                          shiny::sidebarPanel(
                            shiny::radioButtons('likeness_type', 'Type',
                                                c('1d (age)'='1d',
                                                  '2d (age and eHf)'='2d',
                                                  'Combine'='combine')),
                            shiny::uiOutput('likeness_samples'),
                            shiny::numericInput('likeness_age_bw', 'Age bandwidth', 30),
                            shiny::uiOutput('likeness_bw'),
                            shiny::downloadButton('download_likeness_table', 'Save Table')
                          ),
                          shiny::mainPanel(
                            shiny::tableOutput('likeness')
                          )
                        )),

                        # Start of O-parameter age tab
                        shiny::tabPanel('1-O', shiny::sidebarLayout(
                          shiny::sidebarPanel(
                            shiny::radioButtons('o_type', 'Type',
                                                c('Age'='age',
                                                  'Model age'='tdm',
                                                  'Combine' ='combine')),
                            shiny::uiOutput('o_samples'),
                            shiny::downloadButton('download_o_table', 'Save Table'),
                            shiny::tags$hr(),
                            shiny::checkboxInput("o_fig", label = "Graphical", value = FALSE),
                            shiny::conditionalPanel(
                              condition = 'input.o_fig == true',
                              shiny::numericInput('o_width', 'Image Width (cm)', 15),
                              shiny::numericInput('o_height', 'Image Height (cm)', 15),
                              shiny::downloadButton('download_o_plot', 'Save Image')
                            )
                          ),
                          shiny::mainPanel(
                            shiny::uiOutput('o_switch')
                          )
                        )),

                        # Start of Reimink tab
                        shiny::tabPanel('Reimink', shiny::sidebarLayout(
                          shiny::sidebarPanel(
                            shiny::uiOutput('reimink_samples'),
                            shiny::numericInput('reimink_step', 'Chord step (My)', 100),
                            shiny::tags$hr(),
                            shiny::numericInput('reimink_width', 'Image Width (cm)', 15),
                            shiny::numericInput('reimink_height', 'Image Height (cm)', 15),
                            shiny::downloadButton('download_reimink_plot', 'Save Image')
                          ),
                          shiny::mainPanel(
                            shiny::plotOutput('reimink_plot'),
                            shiny::tags$hr(),
                            shiny::tags$p('Lower: '),
                            shiny::verbatimTextOutput('reimink_maxima_lower'),
                            shiny::tags$p('Upper: '),
                            shiny::verbatimTextOutput('reimink_maxima_upper'),
                            shiny::tags$hr(),
                            shiny::downloadButton('download_reimink_table', 'Save likelihood data')
                          )
                        )),
                        shiny::navbarMenu('More',
                        # Start of Constants tab
                        shiny::tabPanel('Constants', shiny::fluidPage(
                          shiny::fluidRow(
                            shiny::column(4,
                                          shiny::numericInput('lambda_lu',
                                                              '176Lu decay constant',
                                                              consts$user$lambda_lu),
                                          shiny::tags$hr(),
                                          shiny::numericInput('luhf_chur',
                                                              '176Lu/177Hf CHUR',
                                                              consts$user$luhf_chur),
                                          shiny::numericInput('hfhf_chur',
                                                              '176Hf/177Hf CHUR',
                                                              consts$user$hfhf_chur),
                                          shiny::tags$hr(),
                                          shiny::numericInput('luhf_dm',
                                                              '176Lu/177Hf DM',
                                                              consts$user$luhf_dm),
                                          shiny::numericInput('hfhf_dm',
                                                              '176hf/177Hf DM',
                                                              consts$user$hfhf_dm),
                                          shiny::numericInput('luhf_zrc',
                                                              '176Lu/177Hf',
                                                              consts$user$luhf_zrc),
                                          shiny::tags$hr(),
                                          shiny::actionButton('reset', 'Reset'),
                                          shiny::actionButton('save', 'Save')
                            )
                          )
                        )),
                        # Start of Plot options tab
                        shiny::tabPanel('Plot options', shiny::fluidPage(
                          shiny::fluidRow(
                            shiny::column(4,
                                          shiny::selectInput('font_name', 'Font',
                                                             if (.Platform$OS.type == 'windows')
                                                               c('Helvetica' = 'sans', 'Courier' = 'mono',
                                                                 'Times'= 'serif')
                                                             else
                                                               c('Helvetica', 'Courier', 'Times')),
                                          shiny::tags$hr(),
                                          shiny::sliderInput('title_size', 'Axes title size (pts)',
                                                             min = 5, max = 20, value = 10),
                                          shiny::sliderInput('label_size', 'Axes label size (pts)',
                                                             min = 5, max = 20, value = 7),
                                          shiny::sliderInput('legend_size', 'Legend text size (pts)',
                                                             min = 5, max = 20, value = 10),
                                          shiny::sliderInput('strip_size', 'Panel text size (pts)',
                                                             min = 5, max = 20, value = 7),
                                          shiny::tags$hr()
                            )
                          )
                        )),
                        shiny::tabPanel('About', shiny::fluidPage(
                          shiny::fluidRow(
                            shiny::column(12, align='center',
                                          shiny::h3(paste('detzrcr', as.character(packageVersion('detzrcr')))),
                                          shiny::tags$br(),
                                          shiny::tags$div(class='header', checked=NA,
                                                          shiny::tags$a(href='https://cran.r-project.org/package=detzrcr',
                                                                        'https://cran.r-project.org/package=detzrcr'),
                                                          shiny::tags$br(),
                                                          shiny::tags$a(href='https://github.com/magnuskristoffersen/detzrcr',
                                                                        'https://github.com/magnuskristoffersen/detzrcr'),
                                                          shiny::tags$br(),
                                                          shiny::tags$a(href='https://cran.r-project.org/web/packages/detzrcr/vignettes/detzrcr-vignette.html',
                                                                        'https://cran.r-project.org/web/packages/detzrcr/vignettes/detzrcr-vignette.html')))
                          ))
                        ))
))

server <- shiny::shinyServer(function(input, output, session) {
  shiny::observeEvent(input$reset, {
    shiny::updateNumericInput(session, 'lambda_lu', value=consts$default$lambda_lu)
    shiny::updateNumericInput(session, 'luhf_chur ', value=consts$default$luhf_chur)
    shiny::updateNumericInput(session, 'hfhf_chur ', value=consts$default$hfhf_chur)
    shiny::updateNumericInput(session, 'luhf_dm', value=consts$default$luhf_dm)
    shiny::updateNumericInput(session, 'hfhf_dm', value=consts$default$hfhf_dm)
    shiny::updateNumericInput(session, 'luhf_zrc', value=consts$default$luhf_zrc)
    })
  shiny::observeEvent(input$save, {
    consts$user$lambda_lu <- input$lambda_lu
    consts$user$luhf_chur <- input$luhf_chur
    consts$user$hfhf_chur <- input$hfhf_chur
    consts$user$luhf_dm <- input$luhf_dm
    consts$user$hfhf_dm <- input$hfhf_dm
    consts$user$luhf_zrc <- input$luhf_zrc
    yaml::write_yaml(consts, file='constants.yml')
  })
  # Reactives
  csv_data <- shiny::reactive({
    numbers <- NULL
    n_conc <- NULL
    n_hf <- NULL
    if (input$example_data == FALSE) {
      inFile <- input$file1
      if (is.null(inFile)) {
        return(NULL)
      }
      n <- dim(inFile)[1]
      if (n == 1) {
        dat <- utils::read.csv(inFile$datapath, header=TRUE, sep=input$sep,
                               quote=input$quote)
      }
      if (n > 1) {
        dat <- utils::read.csv(inFile[[1, 'datapath']], header=TRUE,
                               sep=input$sep, quote=input$quote)
        for (i in seq(2, n)) {
          csv <- utils::read.csv(inFile[[i, 'datapath']], header=TRUE,
                          sep=input$sep, quote=input$quote)
          dat <- merge(dat, csv, all = TRUE)

        }
      }
    } else {
      dat <- utils::read.csv(system.file("extdata", "Natal_group.csv",
                                           package="detzrcr"))
    }
    names(dat) <- tolower(names(dat))
    if ('sample' %in% names(dat)) {
      dat$sample <- as.factor(dat$sample)
    }
    n_analyses <- aggregate(dat$sample, by = dat['sample'], length)
    if (input$disc) {
      nas <- NULL
      if ((is.character(dat$disc)) | (is.factor(dat$disc))) {
        dat$disc <- suppressWarnings(as.numeric(as.character(dat$disc)))
        nas <- which(is.na(dat$disc))
        dat <- dat[-nas, ]
        nas_txt <- paste(nas, collapse=', ')
        nas_txt <- paste('Removed row(s)', nas_txt, 'because disc-column
                         contains non-numeric values', sep=' ')
        output$nas <- shiny::renderPrint({
          cat(nas_txt)
        })
      }
      if (!is.null(input$disc_limit)) {
        dat <- check_conc(dat, disc_lim=input$disc_limit)
      }
      if (is.null(nas)) {
        output$nas <- shiny::renderPrint({
          cat('')
        })
      }
    }
    if (input$sample_freq) {
      if ('hfhf' %in% names(dat)) {
        hf <- dat[which(!is.na(dat$hfhf)), ]
        n_hf <- aggregate(hf$sample, by = hf['sample'], length)
      }
      if ('ehf_i' %in% names(dat)) {
        hf <- dat[which(!is.na(dat$ehf_i)), ]
        n_hf <- aggregate(hf$sample, by = dat['sample'], length)
      }
      if (!is.null(input$disc_limit)) {
        dat1 <- check_conc(dat, disc_lim=input$disc_limit)
        n_conc <- aggregate(dat1$sample, by = dat1['sample'], length)
      }
      if (is.null(n_conc) &
          is.null(n_hf)) {
        output$numbers <- shiny::renderTable({
          names(n_analyses) <- c('sample', 'n_analyses')
          n_analyses
        })
      }
      if (!is.null(n_conc) & (is.null(n_hf))) {
        output$numbers <- shiny::renderTable({
          numbers <- merge(n_analyses, n_conc, by = 'sample')
          names(numbers) <- c('sample', 'n_analyses', 'n_conc')
          numbers
        })
      }
      if (!is.null(n_hf) & is.null(n_conc)) {
        output$numbers <- shiny::renderTable({
          numbers <- merge(n_analyses, n_hf, by = 'sample')
          names(numbers) <- c('sample', 'n_analyses', 'n_hf')
          numbers
        })
      }
      if (!is.null(n_hf) & !is.null(n_conc)) {
        output$numbers <- shiny::renderTable({
          numbers <- merge(n_analyses, n_conc, by = 'sample')
          numbers <- merge(numbers, n_hf, by = 'sample')
          names(numbers) <- c('sample', 'n_analyses', 'n_conc', 'n_hf')
          numbers
        })
      }
    } else {
      numbers <- NULL
      output$numbers <- shiny::renderTable({
        numbers
      })
    }
    return(dat)
  })
  likeness_table <- shiny::reactive({
    new_data <- csv_data()
    if (!is.null(new_data)) {
      constants <- c(
        input$lambda_lu,
        input$hfhf_chur,
        input$luhf_chur,
        input$hfhf_dm,
        input$luhf_dm,
        input$luhf_zrc
      )
      if(!is.null(input$likeness_samples)) {
        new_data <- new_data[new_data$sample %in% input$likeness_samples, ]
        new_data$sample <- factor(new_data$sample,
                                  levels=input$likeness_samples)
      }
      if (input$likeness_type == '1d') {
        satkoski_1d_matrix(new_data, bw=input$likeness_age_bw)
      } else {
        if (input$likeness_type == '2d') {
          hf_data <- calc_hf(new_data, constants=constants)
          satkoski_2d_matrix(hf_data, bw=c(input$likeness_age_bw,
                                           input$likeness_ehf_bw))
        } else {
          if (input$likeness_type == 'combine') {
            hf_data <- calc_hf(new_data, constants=constants)
            one_d <- satkoski_1d_matrix(new_data, bw=input$likeness_age_bw)
            two_d <- satkoski_2d_matrix(hf_data, bw=c(input$likeness_age_bw,
                                                   input$likeness_ehf_bw))
            combine_matrices(one_d, two_d)
          }
        }
      }
    }
  })

  o_table <- shiny::reactive({
    new_data <- csv_data()
    if (!is.null(new_data)) {
      constants <- c(
        input$lambda_lu,
        input$hfhf_chur,
        input$luhf_chur,
        input$hfhf_dm,
        input$luhf_dm,
        input$luhf_zrc
      )
      if(!is.null(input$o_samples)) {
        new_data <- new_data[new_data$sample %in% input$o_samples, ]
        new_data$sample <- factor(new_data$sample,
                                  levels=input$o_samples)
      }
      if (input$o_type == 'age') {
        o_param_matrix_age(new_data)

      } else {
        if (input$o_type == 'tdm') {
          hf_data <- calc_hf(new_data, constants=constants)
          o_param_matrix_tdm(hf_data)
        } else {
          if (input$o_type == 'combine') {
            hf_data <- calc_hf(new_data, constants=constants)
            combine_matrices(o_param_matrix_age(new_data),
                             o_param_matrix_tdm(hf_data))
          }
        }
      }
    }
  })

  o_plot <- shiny::reactive({
    new_data <- csv_data()
    if (!is.null(new_data)) {
      constants <- c(
        input$lambda_lu,
        input$hfhf_chur,
        input$luhf_chur,
        input$hfhf_dm,
        input$luhf_dm,
        input$luhf_zrc
      )
      if ('hfhf' %in% names(new_data)) {
        hf_data <- calc_hf(new_data, constants=constants)
      }
      if(!is.null(input$o_samples)) {
        new_data <- new_data[new_data$sample %in% input$o_samples,]
        new_data$sample <- factor(new_data$sample,
                                  levels=input$o_samples)

        if ('hfhf' %in% names(new_data)) {
          hf_data <- hf_data[hf_data$sample %in% input$o_samples, ]
          hf_data$sample <- factor(hf_data$sample,
                                    levels=input$o_samples)
        }

      }
      if ('hfhf' %in% names(new_data)) {
        dat <- list(age=new_data, hf=hf_data)
      } else {
        dat <- list(age=new_data)
      }
      plot_tile(dat, type=input$o_type) +
        plot_text_options(font_name = input$font_name,
                          title_size = input$title_size,
                          label_size = input$label_size,
                          legend_size = input$legend_size,
                          strip_text_y_size = input$strip_size)
    }
  })

  reimink_plot <- shiny::reactive({
    new_data <- reimink_table()
    if (!is.null(new_data)) {
      plot_reimink(new_data) +
        plot_text_options(font_name = input$font_name,
                          title_size = input$title_size,
                          label_size = input$label_size,
                          legend_size = input$legend_size,
                          strip_text_y_size = input$strip_size)
    }
  })

  reimink_table <- shiny::reactive({
    new_data <- csv_data()
    if (!is.null(new_data)) {
      if(!is.null(input$reimink_samples)) {
        new_data <- new_data[new_data$sample %in% input$reimink_samples, ]
        new_data$sample <- factor(new_data$sample,
                                  levels=input$reimink_samples)
      }
      reimink_data <- reimink(new_data, input$reimink_step)
    }
  })

  hf_table <- shiny::reactive({
    new_data <- csv_data()
    constants <- c(
      input$lambda_lu,
      input$hfhf_chur,
      input$luhf_chur,
      input$hfhf_dm,
      input$luhf_dm,
      input$luhf_zrc
    )
    hf_data <- calc_hf(new_data, constants=constants)
  })
  dens_plot <- shiny::reactive({
    new_data <- csv_data()
    if (!is.null(new_data)) {
       facet <- NULL
      if (input$dens_type == 'dens_combine') {
        facet <- FALSE
        if (!is.null(input$dens_combine_choice)) {
          new_data <- new_data[(new_data$sample %in%
                                  input$dens_combine_choice) ,]
        }
      }
      if (input$dens_type == 'dens_facet') {
        facet <- TRUE
        if (!is.null(input$dens_facet_choice)) {
          new_data <- new_data[(new_data$sample %in% input$dens_facet_choice), ]
          new_data$sample <- factor(new_data$sample,
                                    levels = input$dens_facet_choice)
        }
      }
      if (input$dens_type == 'dens_ind') {
        facet <- FALSE
        if (!is.null(input$dens_ind_choice)) {
          new_data <- new_data[new_data$sample == input$dens_ind_choice, ]
        }
      }
       if (input$hist == TRUE) {
         p <- plot_dens_hist(new_data, binwidth=input$binwidth, bw=input$bw,
                             type=input$type, age_range=c(input$xstart, input$xstop),
                             facet=facet, fixed_y=input$fixed_y,
                             step=input$xstep) +
           plot_text_options(font_name = input$font_name,
                             title_size = input$title_size,
                             label_size = input$label_size,
                             legend_size = input$legend_size,
                             strip_text_y_size = input$strip_size)
       } else {
         p <- plot_dens(new_data, bw=input$bw,
                        type=input$type, age_range=c(input$xstart, input$xstop),
                        facet=facet, fixed_y=input$fixed_y, step=input$xstep) +
           plot_text_options(font_name = input$font_name,
                             title_size = input$title_size,
                             label_size = input$label_size,
                             legend_size = input$legend_size,
                             strip_text_y_size = input$strip_size)
       }
    }
  })
  ecdf_plot <- shiny::reactive({
    new_data <- csv_data()
    if (!is.null(new_data)) {
      mult_ecdf <- NULL
      constants <- c(
        input$lambda_lu,
        input$hfhf_chur,
        input$luhf_chur,
        input$hfhf_dm,
        input$luhf_dm,
        input$luhf_zrc
      )
      if (input$ecdf_input_type == 't_dm2') {
        new_data <- calc_hf(new_data, constants=constants)
      }
      if (input$ecdf_type == 'ind_plot') {
        mult_ecdf <- FALSE
        if (!is.null(input$ecdf_ind_samples)) {
          new_data <- new_data[new_data$sample == input$ecdf_ind_samples, ]
        }
      }
      if (input$ecdf_type == 'same_plot') {
        mult_ecdf <- TRUE
        if (length(input$ecdf_mult_samples) > 0) {
          new_data <- new_data[new_data$sample %in% input$ecdf_mult_samples, ]
        }
      }
      if (input$ecdf_type == 'ecdf_combine_plot') {
        mult_ecdf <- FALSE
        if (!is.null(input$ecdf_comb_samples)) {
          new_data <- new_data[new_data$sample %in% input$ecdf_comb_samples, ]
        }
      }
      p <- plot_ecdf(new_data, mult_ecdf=mult_ecdf, conf=input$ecdf_conf,
                     column=input$ecdf_input_type, guide=input$ecdf_legend) +
        plot_axis_lim(xlim=c(input$ecdf_xstart, input$ecdf_xstop),
                      step=input$ecdf_xstep) +
        plot_text_options(font_name = input$font_name,
                            title_size = input$title_size,
                            label_size = input$label_size,
                            legend_size = input$legend_size)
    }
  })
  hf_plot <- shiny::reactive({
    new_data <- csv_data()
    if (!is.null(new_data)) {
      constants <- c(
        input$lambda_lu,
        input$hfhf_chur,
        input$luhf_chur,
        input$hfhf_dm,
        input$luhf_dm,
        input$luhf_zrc
      )
      new_data <- calc_hf(new_data, constants=constants)
      if (input$hf_type == 'ehf_plot') {
        contour_data <- NULL
        if (input$error_bars) {
          if (input$x_error_bars) {
            x_errors <- data.frame(xmin=new_data$age-(2*new_data$uncert),
                                   xmax=new_data$age+(2*new_data$uncert))
            new_data <- cbind(new_data, x_errors)
          }
          if (input$y_error_bars) {
            y_errors <- data.frame(ymin=new_data$ehf_i-new_data$ehf_2se,
                                   ymax=new_data$ehf_i+new_data$ehf_2se)
            new_data <- cbind(new_data, y_errors)
          }
        }
        if (input$add_contours) {
          contour_data <- new_data[new_data$sample %in% input$contour_choice, ]
        }
        if(!is.null(input$hfhf_samples)) {
          new_data <- new_data[new_data$sample %in% input$hfhf_samples, ]
          new_data$sample <- factor(new_data$sample,
                                    levels=input$hfhf_samples)
        }
        p <- plot_hf(new_data, guide=input$hf_legend,
                     contours=input$add_contours,
                     x_bandwidth=input$contour_x_bandwidth,
                     y_bandwidth=input$contour_y_bandwidth,
                     contour_data=contour_data,
                     combine_contours=input$combine_contours,
                     error_bars=input$error_bars,
                     x_errors=input$x_error_bars, y_errors=input$y_error_bars,
                     constants=constants) +
          plot_axis_lim(xlim=c(input$hf_xstart, input$hf_xstop),
                        ylim=input$ehf_ylim,
                        step=input$hf_xstep) +
          plot_text_options(font_name = input$font_name,
                            title_size = input$title_size,
                            label_size = input$label_size,
                            legend_size = input$legend_size)
      } else {
        if (input$hf_type == 'hfhf_plot') {
          contour_data <- NULL
          if (input$error_bars) {
            if (input$x_error_bars) {
              x_errors <- data.frame(xmin=new_data$age-(2*new_data$uncert),
                                     xmax=new_data$age+(2*new_data$uncert))
              new_data <- cbind(new_data, x_errors)
            }
            if (input$y_error_bars) {
              y_errors <- data.frame(ymin=new_data$hf_i-new_data$hf_2se,
                                     ymax=new_data$hf_i+new_data$hf_2se)
              new_data <- cbind(new_data, y_errors)
            }
          }
          if (input$add_contours) {
            contour_data <-
              new_data[new_data$sample %in% input$contour_choice, ]
          }
          if(!is.null(input$hfhf_samples)) {
            new_data <- new_data[new_data$sample %in% input$hfhf_samples, ]
            new_data$sample <- factor(new_data$sample, levels=
                                        input$hfhf_samples)
          }
          p <- plot_hf(new_data, plot_type = 'hfhf', guide=input$hf_legend,
                       contours=input$add_contours,
                       x_bandwidth=input$contour_x_bandwidth,
                       y_bandwidth=input$contour_y_bandwidth,
                       contour_data=contour_data,
                       combine_contours=input$combine_contours,
                       error_bars=input$error_bars,
                       x_errors=input$x_error_bars, y_errors=input$y_error_bars,
                       constants=constants) +
            plot_axis_lim(xlim=c(input$hf_xstart, input$hf_xstop),
                          ylim=input$hf_ylim,
                          step=input$hf_xstep) +
            plot_text_options(font_name = input$font_name,
                                title_size = input$title_size,
                                label_size = input$label_size,
                                legend_size = input$legend_size)
        }
      }
    }
  })

  uqlq_plot <- shiny::reactive({
    new_data <- csv_data()
    if (!is.null(new_data)) {
      constants <- c(
        input$lambda_lu,
        input$hfhf_chur,
        input$luhf_chur,
        input$hfhf_dm,
        input$luhf_dm,
        input$luhf_zrc
      )
      mix_data = NULL
      if (input$mixing_model) {
        mix_data <- dzr_mix(input$mu1, input$sig1, input$mu2, input$sig2)
      }
      if(!is.null(input$quant_samples)) {
        new_data <- new_data[new_data$sample %in% input$quant_samples, ]
        new_data$sample <- factor(new_data$sample,
                                  levels=input$quant_samples)
      }
      if (input$uqlq_type == 'uqlq_age') {
        column = 'age'
      }
      if (input$uqlq_type == 'uqlq_tdm') {
        new_data <- calc_hf(new_data, constants=constants)
        column = 't_dm2'
      }
      p <- plot_quantiles(new_data, column=column, guide=input$uqlq_legend,
                          conf=input$uqlq_conf, mix=input$mixing_model,
                          mix_data=mix_data)
      p <- p + plot_axis_lim(xlim=c(input$uqlq_xstart, input$uqlq_xstop),
                             step=input$uqlq_xstep,
                             ylim=c(input$uqlq_ystart, input$uqlq_ystop))
      p <- p + plot_text_options(font_name = input$font_name,
                                 title_size = input$title_size,
                                 label_size = input$label_size,
                                 legend_size = input$legend_size)
    }
  })
  # Output
  output$head <- DT::renderDataTable(DT::datatable({
    new_data <- csv_data()
    new_data
  }))

  output$dens_plot <- shiny::renderPlot({
    print(dens_plot())
  })
  output$ecdf_plot <- shiny::renderPlot({
    print(ecdf_plot())
  })
  output$reimink_plot <- shiny::renderPlot({
    print(reimink_plot())
  })
  output$downloadDensplot <- shiny::downloadHandler(
    filename = function(){
      paste('kde-', format(Sys.time(), "%d%m%y-%H%M%S"), '.pdf', sep='')

    },
    content = function(file) {
      ggplot2::ggsave(file, plot = dens_plot(), width=input$densWidth,
                      height=input$densHeight, colormodel='cmyk', units='cm')
    }
  )
  output$download_ecdf_plot <- shiny::downloadHandler(
    filename = function(){
      paste('ecdf-', format(Sys.time(), "%d%m%y-%H%M%S"), '.pdf', sep='')

    },
    content = function(file) {
      ggplot2::ggsave(file, plot = ecdf_plot(), width=input$ecdf_width,
                      height=input$ecdf_height, colormodel='cmyk', units='cm')
    }
  )
  output$download_hf_plot <- shiny::downloadHandler(
    filename = function(){
      paste('hf-', format(Sys.time(), "%d%m%y-%H%M%S"), '.pdf', sep='')

    },
    content = function(file) {
      ggplot2::ggsave(file, plot = hf_plot(), width=input$hf_width,
                      height=input$hf_height, colormodel='cmyk', units='cm',
                      useDingbats=FALSE)
    }
  )
  output$download_uqlq_plot <- shiny::downloadHandler(
    filename = function(){
      paste('uqlq-', format(Sys.time(), "%d%m%y-%H%M%S"), '.pdf', sep='')

    },
    content = function(file) {
      ggplot2::ggsave(file, plot = uqlq_plot(), width=input$uqlq_width,
                      height=input$uqlq_height, colormodel='cmyk', units='cm',
                      useDingbats=FALSE)
    }
  )
  output$download_likeness_table <- shiny::downloadHandler(
    filename = function(){
      paste('likeness-', format(Sys.time(), "%d%m%y-%H%M%S"), '.csv', sep='')

    },
    content = function(file) {
      utils::write.csv(likeness_table(), file, row.names = FALSE)
    }
  )
  output$download_satkoski_2d <- shiny::downloadHandler(
    filename = function(){
      paste('L2-', format(Sys.time(), "%d%m%y-%H%M%S"), '.csv', sep='')

    },
    content = function(file) {
      utils::write.csv(satkoski_2d_table(), file, row.names = FALSE)
    }
  )
  output$download_o_table <- shiny::downloadHandler(
    filename = function(){
      paste('otable-', format(Sys.time(), "%d%m%y-%H%M%S"), '.csv', sep='')

    },
    content = function(file) {
      utils::write.csv(o_table(), file, row.names = FALSE)
    }
  )
  output$download_o_plot <- shiny::downloadHandler(
    filename = function(){
      paste('oplot-', format(Sys.time(), "%d%m%y-%H%M%S"), '.pdf', sep='')

    },
    content = function(file) {
      ggplot2::ggsave(file, plot = o_plot(), width=input$o_width,
                      height=input$o_height, colormodel='cmyk', units='cm')
    }

  )
  output$download_reimink_plot <- shiny::downloadHandler(
    filename = function(){
      paste('reimink-', format(Sys.time(), "%d%m%y-%H%M%S"), '.pdf', sep='')

    },
    content = function(file) {
      ggplot2::ggsave(file, plot = reimink_plot(), width=input$reimink_width,
                      height=input$reimink_height, colormodel='cmyk',
                      units='cm')
    }
  )
  output$download_reimink_table <- shiny::downloadHandler(
    filename = function(){
      paste('likelihood-', format(Sys.time(), "%d%m%y-%H%M%S"), '.csv', sep='')

    },
    content = function(file) {
      utils::write.csv(reimink_table(), file, row.names = FALSE)
    }
  )
  output$download_hf_table <- shiny::downloadHandler(
    filename = function(){
      paste('hf-', format(Sys.time(), "%d%m%y-%H%M%S"), '.csv', sep='')

    },
    content = function(file) {
      utils::write.csv(hf_table(), file, row.names = FALSE)
    }
  )
  output$dens_facet_select <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    selectInput('dens_facet_choice', 'Select samples', samples,
                multiple=TRUE, selectize=TRUE)
  })
  output$show_disc_limit <- shiny::renderUI({
    if (input$disc) {
      shiny::numericInput('disc_limit', 'Discordancy limit (%)', 10,
                          min=0, max=100)
      }
  })
  # Dynamic UI density tab
  output$dens_switch <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    if (input$dens_type == 'dens_facet') {
      selectInput('dens_facet_choice', 'Select samples', samples,
                  multiple=TRUE, selectize=TRUE)
    } else {
      if (input$dens_type == 'dens_ind') {
        selectInput('dens_ind_choice', 'Select sample', samples)
      } else {
      if (input$dens_type == 'dens_combine') {
        selectInput('dens_combine_choice', 'Select samples', samples,
                    multiple=TRUE, selectize=TRUE)
      }
      }
    }
  })
  # Dynamic UI ecdf tab
  output$ecdf_switch <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    if (input$ecdf_type == 'same_plot') {
      selectInput('ecdf_mult_samples', 'Select samples', samples,
                  multiple=TRUE, selectize=TRUE)
    } else {
      if (input$ecdf_type == 'ind_plot') {
        selectInput('ecdf_ind_samples', 'Select sample', samples)
      } else {
        if (input$ecdf_type == 'ecdf_combine_plot') {
          selectInput('ecdf_comb_samples', 'Select samples', samples,
                      multiple=TRUE, selectize=TRUE)
        }
      }
    }
  })
  # Dynamic UI Lu-Hf tab
  output$hf_switch <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    if (input$hf_type == 'ehf_plot') {
      shiny::sliderInput('ehf_ylim', 'Y-axis range',
                         min=-50, max=50, value = c(-20, 20))
    } else {
      if (input$hf_type == 'hfhf_plot') {
        shiny::sliderInput('hf_ylim', 'Y-axis range',
                           min=0.279, max=0.2833, value = c(0.28, 0.2831))
        }
      }
  })
  output$hf_bandwidths <- shiny::renderUI({
    if (input$hf_type == 'ehf_plot') {
      shiny::conditionalPanel(
        condition = 'input.add_contours == true',
        shiny::numericInput('contour_x_bandwidth', 'X bandwidth', 30),
        shiny::numericInput('contour_y_bandwidth', 'Y bandwidth', 2.5),
        shiny::checkboxInput('combine_contours',
                             label = 'Combine contours',
                             value = FALSE)
      )
    } else {
      if (input$hf_type == 'hfhf_plot') {
        shiny::conditionalPanel(
          condition = 'input.add_contours == true',
          shiny::numericInput('contour_x_bandwidth', 'X bandwidth', 30),
          shiny::numericInput('contour_y_bandwidth', 'Y bandwidth', 0.00025),
          shiny::checkboxInput('combine_contours',
                               label = 'Combine contours',
                               value = FALSE)
        )
      }
    }
  })
  # Dynamic UI O-tab
  output$o_switch <- shiny::renderUI({
    if (input$o_fig) {
      shiny::verticalLayout(
        shiny::tableOutput('o_table'),
        shiny::plotOutput('o_plot')
      )
    } else {
      shiny::tableOutput('o_table')
    }
  })
  output$contour_switch <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    if (input$add_contours) {
      selectInput('contour_choice', 'Select samples to contour', samples,
                  multiple=TRUE, selectize=TRUE)
    }
  })
  output$hf_samples <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    shiny::selectInput('hfhf_samples', 'Select samples', samples,
                       multiple=TRUE, selectize=TRUE)
  })
  output$hf <- shiny::renderPlot({
    print(hf_plot())
  })
  output$uqlq_samples <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    shiny::selectInput('quant_samples', 'Select samples', samples,
                       multiple=TRUE, selectize=TRUE)
  })
  output$o_samples <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    shiny::selectInput('o_samples', 'Select samples', samples,
                       multiple=TRUE, selectize=TRUE)
  })
  output$likeness_samples <- shiny::renderUI({
    constants <- c(
      input$lambda_lu,
      input$hfhf_chur,
      input$luhf_chur,
      input$hfhf_dm,
      input$luhf_dm,
      input$luhf_zrc
    )
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    shiny::selectInput('likeness_samples', 'Select samples', samples,
                       multiple=TRUE, selectize=TRUE)
  })
  output$uqlq <- shiny::renderPlot({
    print(uqlq_plot())
  })
  output$likeness <- shiny::renderTable({
    likeness_table()
  }, rownames=TRUE)
  output$likeness_bw <- shiny::renderUI({
    if (input$likeness_type == '2d' | input$likeness_type == 'combine') {
      shiny::numericInput('likeness_ehf_bw', 'Epsilon-Hf bandwidth', 2.5)
    }
  })
  output$o_plot <- shiny::renderPlot({
    print(o_plot())
  })
  output$o_table <- shiny::renderTable({
    o_table()
  }, rownames=TRUE)

  output$reimink_samples <- shiny::renderUI({
    new_data <- csv_data()
    samples <- as.vector(unique(new_data$sample))
    shiny::selectInput('reimink_samples', 'Select samples', samples,
                       multiple=FALSE, selectize=FALSE)
  })
  output$reimink_maxima_lower <- shiny::renderText({
    reimink_data <- reimink_table()
    if (!is.null(reimink_data)) {
        lower <- reimink_data[reimink_data$type == 'lower',]
        lower_maxima <- find_maxima(lower$y, 0, 1)
        lower[lower_maxima, ]$x
    }
  })
  output$reimink_maxima_upper <- shiny::renderText({
    reimink_data <- reimink_table()
    if (!is.null(reimink_data)) {
        upper <- reimink_data[reimink_data$type == 'upper',]
        upper_maxima <- find_maxima(upper$y, 0, 1)
        upper[upper_maxima, ]$x
    }
  })
})

# Run the application
shiny::shinyApp(ui = ui, server = server)

#' Launch shiny interface
#'
#'@param ... Pass arguments on to shiny::runApp
#' @export
#'
run_detzrcr <- function(...) {
  app_dir <- system.file('shiny-apps', 'detzrcr_app', package = 'detzrcr')
  shiny::runApp(app_dir, display.mode = "normal", ...)
}

#' Dataset Natal group
#'
#' Detrital zircon U-Pb age and Lu-Hf data from the Natal Group, KwaZulu-Natal,
#' South-Africa (Kristoffersen et al. 2016).
#'
#' Used as example of proper format of a csv-file for input in the shiny
#' interface.
#'
#' @name Natal_group
#'
#'
#' @references Kristoffersen, M., Andersen, T., Elburg, M.A., Watkeys, M.K.,
#' 2016. Detrital zircon in a supercontinental setting: locally derived and
#' far-transported components in the Ordovician Natal Group, South Africa.
#' J. Geol. Soc. London. 173, 203-215. doi:10.1144/jgs2015-012
#'
NULL

utils::globalVariables('.data')
#' Plot 1d density
#'
#' Creates density for U-Pb data.
#'
#' @param dat data.frame
#' @param bw density bandwidth
#' @param type 'kde': traditional KDE 'pdd': detrital zircon PDD
#' @param age_range range over which to calculate density
#' @param facet logical, facet samples?
#' @param fixed_y logical, fixed y-axis?
#' @param step specify x-axis steps
#' @export
#' @return  ggplot2 1d density plot with histogram
plot_dens <- function(dat, bw=30, type='kde',
                           age_range=c(0, 4560),
                           facet=FALSE, fixed_y=FALSE, step=200) {
  if (facet) {
    l <- lapply(split(dat, factor(dat$sample)), calc_dens, bw=bw,
                type=type, age_range=age_range)
    dens <- do.call(rbind.data.frame, l)
    gplot <- ggplot2::ggplot()
    gplot <- gplot + ggplot2::geom_path(data=dens,
                                        ggplot2::aes(x=.data$x, y=.data$y),
                                        na.rm=TRUE)
      if (fixed_y) {
        gplot <- gplot + ggplot2::facet_grid(sample ~ .)
      } else {
        gplot <- gplot + ggplot2::facet_grid(sample ~ ., scale='free_y')
      }
    gplot <- gplot + plot_labels(ylab = 'Density') + plot_bw_theme() +
      plot_axis_lim(xlim=age_range, step=step)
    return(gplot)
  } else {
    dens <- calc_dens(dat, bw=bw, type=type,
                           age_range=age_range)
    gplot <- ggplot2::ggplot()
    gplot <- gplot + ggplot2::geom_path(data=dens,
                                        ggplot2::aes(x=.data$x, y=.data$y),
                                        na.rm=TRUE)
    gplot <- gplot + plot_labels(ylab = 'Density') + plot_bw_theme() +
      plot_axis_lim(xlim=age_range, step=step)
    return(gplot)
  }
}

#' Plot 1d density with histogram
#'
#' Creates density and histogram plot of U-Pb data.
#'
#' @param dat data.frame
#' @param bw density bandwidth
#' @param binwidth histogram binwidth
#' @param type 'kde': traditional KDE 'pdd': detrital zircon PDD
#' @param age_range range over which to calculate density
#' @param facet logical, facet samples?
#' @param fixed_y logical, fixed y-axis?
#' @param step specify x-axis steps
#' @export
#' @return  ggplot2 1d density plot with histogram
plot_dens_hist <- function(dat, bw=30, binwidth=50, type='kde',
                              age_range=c(0, 4560), facet=FALSE,
                              fixed_y=FALSE, step=200) {
  if (facet) {
    l <- lapply(split(dat, factor(dat$sample)), calc_dens_hist, bw=bw,
                binwidth=binwidth, type=type, age_range=age_range)
    dens <- do.call(rbind.data.frame, l)
    gplot <- ggplot2::ggplot() + ggplot2::geom_histogram(
      data=dat,
      ggplot2::aes(x=.data$age),
      color='grey90',
      fill='grey60',
      breaks=seq(0, 4560,
                 binwidth),
      na.rm=TRUE)
      if (fixed_y) {
        gplot <- gplot + ggplot2::facet_grid(sample ~ .)
      } else {
      gplot <- gplot + ggplot2::facet_grid(sample ~ ., scale='free_y')
        }
    gplot <- gplot + ggplot2::geom_path(data=dens,
                                        ggplot2::aes(x=.data$x,
                                                     y=.data$y),
                                        na.rm=TRUE)
    gplot <- gplot + plot_labels(ylab = 'Count') + plot_bw_theme() +
      plot_axis_lim(xlim=age_range, step=step)
    return(gplot)
    } else {
      dens <- calc_dens_hist(dat, binwidth=binwidth, bw=bw, type=type,
                             age_range=age_range)
      gplot <- ggplot2::ggplot() + ggplot2::geom_histogram(
        data=dat,
        ggplot2::aes(x=.data$age),
        color='grey90',
        fill='grey60',
        breaks=seq(0, 4560,
                   binwidth), na.rm=TRUE)
      gplot <- gplot + ggplot2::geom_path(data=dens,
                                          ggplot2::aes(x=.data$x, y=.data$y))
      gplot <- gplot + plot_labels(ylab = 'Count') + plot_bw_theme() +
        plot_axis_lim(xlim=age_range, step=step)
      return(gplot)
  }
}

#' Labels for ggplot2 plots
#'
#' Convenience function to label ggplot2
#'
#' @param xlab X-axis label
#' @param ylab Y-axis label
#' @export
#' @return Returns ggplot2 labels
plot_labels <- function(xlab = 'Age (Ma)', ylab = 'Density') {
   p_labels <- list(ggplot2::xlab(xlab),
                    ggplot2::ylab(ylab))
   return(p_labels)
}

#' Axes limits for ggplot2
#'
#' Convenience function to change axes limits for ggplot2
#'
#' @param xlim x-axis limit
#' @param ylim y-axis limit
#' @param step specify x-axis steps
#' @return list of ggplot2::coord_cartesian object
#' @export
#'
plot_axis_lim <- function(xlim = c(0, 4560), step=200, ylim=NULL) {
  p_x_lim <- list(ggplot2::coord_cartesian(xlim=xlim, ylim=ylim),
                  ggplot2::scale_x_continuous(
                                           breaks=seq(xlim[1], xlim[2], step)))
}

#' Modify text options of plots
#'
#' @param font_name Name of font to use
#' @param title_size Font size of x- and y-axis titles
#' @param label_size Font size of x- and y-axis tick labels
#' @param legend_size Font size of legend
#' @param strip_text_y_size Font size of vertical panel text
#'
#' @export
#'
plot_text_options <- function(font_name = 'Helvetica', title_size = 10,
                              label_size = 7, legend_size = 10,
                              strip_text_y_size = 8) {
  text_options <- list(ggplot2::theme(
    text=ggplot2::element_text(family=font_name),
    axis.title=ggplot2::element_text(size=title_size),
    axis.text=ggplot2::element_text(size=label_size),
    legend.text=ggplot2::element_text(size=legend_size),
    legend.title=ggplot2::element_blank(),
    strip.text.y=ggplot2::element_text(size=strip_text_y_size)))
}

#' Stripped down theme for ggplot2
#' @export
#' @return Returns ggplot2 theme
plot_bw_theme <- function() {
  p_bw_theme <- ggplot2::theme_bw() +
    ggplot2::theme(panel.grid.major=ggplot2::element_blank(),
                         panel.grid.minor=ggplot2::element_blank())
  return(p_bw_theme)
}

#' Plot ecdf
#'
#' Plot ecdf for U-Pb age or Lu-Hf model age data.
#'
#' @param dat data.frame
#' @param mult_ecdf logical, plot several ecdfs in same plot
#' @param column which column to use
#' @param conf logical, plot confidence bands
#' @param guide logical, show legend
#' @param alpha alpha-level used for confidence bands
#'
#' @export
#'
plot_ecdf <- function(dat, mult_ecdf=FALSE, column='age', conf=FALSE,
                      guide=TRUE, alpha=0.05) {
  if (column == 't_dm2') {
    dat <- dat[!is.na(dat$ehf_i), ]
  }
  if (mult_ecdf) {
    l <- lapply(split(dat, factor(dat$sample)), calc_dkw, column=column,
                alpha=alpha)
    conf_data <- do.call(rbind.data.frame, l)
    gplot <- ggplot2::ggplot() +
      ggplot2::geom_step(data=conf_data,
                         ggplot2::aes(x=.data$x, y=.data$y,
                                             color=.data$sample), na.rm=TRUE) +
      plot_bw_theme() + plot_labels(ylab = 'Probability')
    if (conf) {
      gplot <- gplot +
        ggplot2::geom_step(data=conf_data,
                           ggplot2::aes(x=.data$x, y=.data$low,
                                        color=.data$sample),
                           linetype=2, na.rm=TRUE) +
        ggplot2::geom_step(data=conf_data,
                           ggplot2::aes(x=.data$x, y=.data$high,
                                        color=.data$sample),
                           linetype=2, na.rm=TRUE)
    }
  } else {
    conf_data <- calc_dkw(dat, column=column, alpha=alpha)
    gplot <- ggplot2::ggplot()
    gplot <- gplot + ggplot2::geom_step(data=conf_data,
                                ggplot2::aes(x=.data$x,
                                             y=.data$y), na.rm=TRUE) +
      plot_bw_theme() + plot_labels(ylab = 'Probability')
    if (conf) {
      gplot <- gplot +
        ggplot2::geom_step(data=conf_data,
                           ggplot2::aes(x=.data$x, y=.data$low), linetype=2,
                           na.rm=TRUE) +
        ggplot2::geom_step(data=conf_data,
                           ggplot2::aes(x=.data$x, y=.data$high), linetype=2,
                           na.rm=TRUE)
    }
  }
  if (guide == FALSE) {
    gplot <- gplot + ggplot2::guides(color="none")
  }
  gplot
}

#' Plot Lu-Hf data
#'
#' Plot Lu-Hf data as both epsilon-Hf vs. age and 176/177Hf vs. age.
#'
#' @param dat data.frame
#' @param range range to display
#' @param plot_type 'ehf'=epsilon-Hf; 'hfhf'=176/177Hf
#' @param guide logical, show legend?
#' @param x_errors logical, include x errorbars
#' @param y_errors logical, include y errorbars
#' @param error_bars logical, include errorbars
#' @param contours logical, plot contours?
#' @param x_bandwidth bandwidth 2dkde x-direction
#' @param y_bandwidth bandwidth 2dkde y-direction
#' @param contour_data data.frame containing data to contour
#' @param combine_contours logical combine contouring data
#' @param constants vector of constants which must be in the order
#' decay constant 176Lu, 176/177Hf CHUR, 176Lu/177Hf CHUR, 176/177Hf DM and
#' 176Lu/177Hf DM
#'
#' @export
#'
plot_hf <- function(dat, range=c(0, 4560), plot_type='ehf', guide=TRUE,
                    x_errors=FALSE, y_errors=FALSE, error_bars=FALSE,
                    contours=FALSE, x_bandwidth=NULL, y_bandwidth=NULL,
                    contour_data=NULL, combine_contours=FALSE,
                    constants) {
  if (plot_type == 'ehf') {
    lines <- hf_lines(range=range, plot_type='ehf', constants)
    gplot <- ggplot2::ggplot()
    gplot <- gplot +
      ggplot2::geom_line(data=lines, ggplot2::aes(x=.data$x, y=.data$line,
                                                   linetype=.data$type), na.rm=TRUE)
    gplot <- gplot + ggplot2::scale_linetype_manual(values=c(1, 1))
    gplot <- gplot + plot_bw_theme() +
      plot_labels(xlab='Age (Ma)',
                  ylab=expression(paste('Initial ', epsilon["Hf"])))
    if (error_bars) {
      if (x_errors) {
        gplot <- gplot + ggplot2::geom_errorbarh(data=dat,
                                                 ggplot2::aes(xmin=.data$xmin,
                                                              xmax=.data$xmax,
                                                              y=.data$ehf_i),
                                                 na.rm=TRUE)
      }
      if (y_errors) {
        gplot <- gplot + ggplot2::geom_errorbar(data=dat,
                                                ggplot2::aes(ymin=.data$ymin,
                                                             ymax=.data$ymax,
                                                             x=.data$age),
                                                na.rm=TRUE)
      }
    }
    if (contours & !is.null(contour_data)) {
      old_names <- c('sample', 'age', 'ehf_i')
      new_names <- c('s', 'a', 'e')
      names(contour_data)[which(names(contour_data) %in% old_names)] <-
        new_names
      if (combine_contours) {
        gplot <- gplot + ggplot2::geom_density2d(data=contour_data,
                                                 ggplot2::aes(x=.data$a,
                                                                     y=.data$e),
                                                 h=c(x_bandwidth,
                                                     y_bandwidth)*4,
                                                 na.rm=TRUE)
          gplot <- gplot + ggplot2::guides(color="none")
      } else {
        gplot <- gplot + ggplot2::geom_density2d(data=contour_data,
                                                 ggplot2::aes(x=.data$a,
                                                              y=.data$e,
                                                              color=.data$s),
                                                 h=c(x_bandwidth,
                                                     y_bandwidth)*4,
                                                 na.rm=TRUE)
        gplot <- gplot + ggplot2::guides(color="none")
      }
    }
    gplot <- gplot +
      ggplot2::geom_point(data=dat,
                          ggplot2::aes(x=.data$age, y=.data$ehf_i,
                                              fill=.data$sample,
                                              shape=.data$sample),
                          color='black',
                          size=3,
                          na.rm=TRUE)
    gplot <- gplot + plot_point_scale()
    gplot <- gplot + ggplot2::guides(linetype="none")
  } else {
    if (plot_type == 'hfhf') {
      lines <- hf_lines(range=range, plot_type='hfhf', constants=constants)
      gplot <- ggplot2::ggplot() +
        ggplot2::geom_line(data=lines, ggplot2::aes(x=.data$x, y=.data$line,
                                                    linetype=.data$type),
                           na.rm=TRUE)
      gplot <- gplot + ggplot2::scale_linetype_manual(values=c(1, 1))
      gplot <- gplot + plot_bw_theme() +
        plot_labels(xlab='Age (Ma)',
                    ylab=expression(paste("Initial "^{176},"Hf/",
                                          ""^{177},"Hf")))
      if (error_bars) {
        if (x_errors) {
          gplot <- gplot + ggplot2::geom_errorbarh(data=dat,
                                                   ggplot2::aes(xmin=.data$xmin,
                                                                xmax=.data$xmax,
                                                                y=.data$hf_i),
                                                   na.rm=TRUE)
        }
        if (y_errors) {
          gplot <- gplot + ggplot2::geom_errorbar(data=dat,
                                                  ggplot2::aes(ymin=.data$ymin,
                                                               ymax=.data$ymax,
                                                               x=.data$age),
                                                  na.rm=TRUE)
        }
      }

      if (contours & !is.null(contour_data)) {
        old_names <- c('sample', 'age', 'hf_i')
        new_names <- c('s', 'a', 'h')
        names(contour_data)[which(names(contour_data) %in% old_names)] <-
          new_names
        if (combine_contours) {
          gplot <- gplot + ggplot2::geom_density2d(data=contour_data,
                                                   ggplot2::aes(x=.data$a,
                                                                y=.data$h),
                                                   h=c(x_bandwidth,
                                                       y_bandwidth)*4,
                                                   na.rm=TRUE) +
            ggplot2::guides(color="none")
        } else {
          gplot <- gplot + ggplot2::geom_density2d(
            data=contour_data,
            ggplot2::aes(x=.data$a, y=.data$h, color=.data$s),
            h=c(x_bandwidth, y_bandwidth)*4, na.rm=TRUE)
          gplot <- gplot + ggplot2::guides(color="none")

        }

      }
      gplot <- gplot +
        ggplot2::geom_point(data=dat,
                            ggplot2::aes(x=.data$age,
                                         y=.data$hf_i,
                                         fill=.data$sample,
                                         shape=.data$sample),
                            color='black', size=3, na.rm=TRUE)
      gplot <- gplot + plot_point_scale()
      gplot <- gplot + ggplot2::guides(linetype="none")
      }
  }
  if (guide == FALSE) {
    gplot <- gplot + ggplot2::guides(fill="none", shape="none")
  }
  gplot
}

#' Plot quantiles
#'
#' @param dat data.frame
#' @param column which column in data.frame to use
#' @param conf logical, plot confidence interval
#' @param alpha alpha-level
#' @param type type of quantile calculation (passed on to stats::quantile)
#' @param guide logical, show legend?
#' @param mix logical, add mixing model
#' @param mix_data mixing model data
#'
#' @export
#'
plot_quantiles <- function(dat, column='t_dm2', conf=FALSE, alpha=0.05, type=8,
                           guide=TRUE, mix=FALSE, mix_data=NULL) {
  quants <- calc_quantiles(dat=dat, column=column, alpha=alpha, type=type)
  line <- data.frame(x=seq(0, 4560), y=seq(0, 4560))
  gplot <- ggplot2::ggplot() +
    ggplot2::geom_line(data=line, ggplot2::aes(x=.data$x, y=.data$y), na.rm=TRUE)
  if (mix & !is.null(mix_data)) {
    gplot <- gplot + ggplot2::geom_line(data=mix_data,
                                        ggplot2::aes(x=.data$lq, y=.data$uq),
                                        color='red', na.rm=TRUE)
  }
  if (conf) {
    l <- lapply(split(dat, factor(dat$sample)), quant_bounds, column=column,
                alpha=alpha)
    quant_data <- do.call(rbind.data.frame, l)
    gplot <- gplot + ggplot2::geom_errorbar(data=quant_data,
                                            ggplot2::aes(x=.data$x,
                                                         ymin=.data$ymin,
                                                         ymax=.data$ymax,
                                                         width=0),
                                            na.rm=TRUE)
    gplot <- gplot + ggplot2::geom_errorbarh(data=quant_data,
                                             ggplot2::aes(y=.data$y,
                                                          xmin=.data$xmin,
                                                          xmax=.data$xmax,
                                                          height=0),
                                             na.rm=TRUE)
  }
  gplot <- gplot +
    ggplot2::geom_point(data=quants, ggplot2::aes(x=.data$twentyfive,
                                                  y=.data$seventyfive,
                                                  fill=.data$sample,
                                                  shape=.data$sample),
                        color='black',
                        size=3,
                        na.rm=TRUE)
  gplot <- gplot + plot_bw_theme() + plot_labels(xlab='Lower quartile (Ma)',
                                                 ylab='Upper quartile (Ma)')
    gplot <- gplot + plot_point_scale()
  if (guide == FALSE) {
    gplot <- gplot + ggplot2::guides(fill="none", shape="none")
  }
  gplot
}

#' Add manual shape scale to scatter plot
#'
#' @export
#'
plot_point_scale <- function() {
  point_scale <- list(ggplot2::scale_shape_manual(values=rep(c(21, 22, 23, 24,
                                                               25), 5)))
}

#' Tile plot of 1-O matrix
#'
#' @param dat data.frame
#' @param type What to plot
#'
#' @export
#'
plot_tile <- function(dat, type) {
  tiles <- make_tiling(dat, type=type)
  gplot <- ggplot2::ggplot(data=tiles) +
    ggplot2::geom_tile(ggplot2::aes(x=.data$x, y=.data$y, fill=.data$z),
                       color='black', na.rm=TRUE) +
    plot_labels(xlab='', ylab='') +
    ggplot2::guides(fill="none") +
    ggplot2::scale_fill_manual(values=c('#1a9641', 'white', '#d7191c'),
                      na.value='grey80', limits=factor(c(0, 1, 2)), drop=FALSE) +
    plot_bw_theme() + ggplot2::theme(axis.text.x =
                                       ggplot2::element_text(angle = 90,
                                                             hjust = 1)) +
    ggplot2::scale_x_discrete(expand=c(0, 0)) +
    ggplot2::scale_y_discrete(expand=c(0, 0))
}

#' Plot likelihood of intercept ages
#'
#' @param dat data.frame
#'
#' @export
#'
plot_reimink <- function(dat) {
  gplot <- ggplot2::ggplot(data=dat, ggplot2::aes(x=.data$x, y=.data$y,
                                                  color=.data$type),
                           na.rm=TRUE) +
    ggplot2::geom_line() + ggplot2::geom_point() + plot_bw_theme() +
    plot_labels(xlab='Age (Ma)', ylab = 'Likelihood')
}

# Added to suppress "Namespace in Imports field not imported from: ‘utils’
# All declared Imports should be used." note in R CMD check
ignore_unused_imports <- function() {
  utils::globalVariables
  DT::renderDataTable
  yaml::write_yaml
}

# Values herein are not used anymore, but will keep the file as a handy
# way of documenting the constants used in calculations.

#' Decay constant of 176Lu.
#'
#' @references Soderlund, U., Jonathan Patchett, P., Vervoort, J.D. and Isachsen
#' C.E. 2004. The 176Lu decay constant determined by Lu^Hf and U^Pb isotope
#' systematics of Precambrian mafic intrusions. Earth and Planetary Science
#' Letters 219, 311-324.

lambda_lu <- 1.867*10^-11

#' 176Lu/177Hf value of CHUR.
#'
#' @references Bouvier, A., Vervoort, J.D. and Jonathan Patchett P. 2008. The
#' Lu-Hf and Sm-Nd isotopic composition of CHUR: Constraints from unequilibrated
#' chondrites and implications for the bulk composition of terrestrial planets.
#' Earth and Planetary Science Letters 273, 48-57.

luhf_chur <- 0.0336

#' 176Hf/177Hf value of CHUR.
#'
#' @references Bouvier, A., Vervoort, J.D. and Jonathan Patchett P. 2008. The
#' Lu-Hf and Sm-Nd isotopic composition of CHUR: Constraints from unequilibrated
#' chondrites and implications for the bulk composition of terrestrial planets.
#' Earth and Planetary Science Letters 273, 48-57.

hfhf_chur <- 0.282785

#' 176Lu/177Hf value of DM.
#'
#' 176Lu/177Hf value of DM (Griffin et al., 2000) recalculated to the decay
#' constant of Soderlund et al. (2004) and the CHUR values of Bouvier et al.
#' (2008).
#'
#' @references Griffin, W., Pearson, N., Belousova, E., Jackson, S., van
#' Achterbergh, E., O'Reilly, S. and Shee, S. 2000. The Hf isotope composition
#' of cratonic mantle:
#' LAM-MC-ICPMS analysis of zircon megacrysts in kimberlites.
#' Geochimica et Cosmochimica Acta 64(1), 133-147.

luhf_dm <- 0.0388

#' 176Lu/177Hf value of DM.
#'
#' @references Griffin, W., Pearson, N., Belousova, E., Jackson, S., van
#' Achterbergh, E., O'Reilly, S. and Shee, S. 2000. The Hf isotope composition
#' of cratonic mantle:
#' LAM-MC-ICPMS analysis of zircon megacrysts in kimberlites.
#' Geochimica et Cosmochimica Acta 64(1), 133-147.

hfhf_dm <- 0.28325

#' 176Lu/177Hf value of average continental crust.
#'
#' @references Griffin, W., Belousova, E., Shee, S., Pearson, N. and O'Reilly,
#' S. 2004. Archean crustal evolution in the northern Yilgam Craton:
#' U-Pb and Hf-isotope evidence from detrital zircons. Precambrian Research,
#' 231-282.

luhf_zrc <- 0.015

#' Decay constants of 235U.
#'
#' @references Steiger, R. & Jager, E. 1977. SUBCOMMISSION ON GEOCHRONOLOGY -
#' CONVENTION ON USE OF DECAY CONSTANTS IN GEOCHRONOLOGY AND COSMOCHRONOLOGY.
#' Earth And Planetary Science Letters 36(3), 359-362.

lambda_u235 <- 9.8485 * (10^-10)

#' Decay constants of 238U.
#'
#' @references Steiger, R. & Jager, E. 1977. SUBCOMMISSION ON GEOCHRONOLOGY -
#' CONVENTION ON USE OF DECAY CONSTANTS IN GEOCHRONOLOGY AND COSMOCHRONOLOGY.
#' Earth And Planetary Science Letters 36(3), 359-362.

lambda_u238 <- 1.55125 * (10^-10)

#' Atomic ratio of 238U and 235U.
#'
#' @references Steiger, R. & Jager, E. 1977. SUBCOMMISSION ON GEOCHRONOLOGY -
#' CONVENTION ON USE OF DECAY CONSTANTS IN GEOCHRONOLOGY AND COSMOCHRONOLOGY.
#' Earth And Planetary Science Letters 36(3), 359-362.

u238_u235_ratio <- 137.88

#' Check concordancy of input ages
#'
#' Check the concordancy of the U-Pb data and return the data within the desired
#' discordancy limit.
#'
#' @param dat data.frame containing at least ages and percentage of discordancy
#' @param disc_lim Discordancy limit
#' @return Concordant data
#' @export
check_conc <- function(dat, disc_lim = 10) {
  if ('disc' %in% names(dat)) {
    llim <- -disc_lim
    ulim <- disc_lim
    conc <- dat[(dat$disc >= llim & dat$disc <= ulim), ]
  } else {
    stop('Requires column with name disc')
  }
}

#' Calculate 1D density of age data
#'
#' Calculate the 1d density of U-Pb age data using KDE or PDD.
#'
#' @param dat data.frame containing at least ages and percentage of discordancy
#' @param bw Bandwidth
#' @param type Type to calculate 'kde': proper KDE; 'pdd': detrital zircon PDD
#' @param age_range Range over which to calculate density
#' @return Density
#' @export
calc_dens <- function(dat, bw=30, type='kde', age_range=c(0, 4560)) {
  n <- age_range[2] - age_range[1]
  if (type == 'kde') {
    dens <- stats::density(dat$age, bw=bw, n=n, from=age_range[1],
                           to=age_range[2])
    x <- dens$x
    y <- dens$y
   }
  if (type == 'pdd') {
    if ('uncert' %in%  names(dat)) {
      x <- seq(age_range[1], age_range[2])
      y <- sapply(x, FUN=function(x) sum(stats::dnorm(x, dat$age, dat$uncert)))
      y <- y / length(x)
    } else {
      stop('Cannot calculate pdd without individual uncertainty for each grain')
    }
  }
  dens <- data.frame(x=x, y=y)
  if(!is.null(dat$sample)) {
    dens$sample <- rep(dat$sample[1], nrow(dens))
  }
  return(dens)
}

#' Calculate scaled 1d density
#'
#' Calculates 1d density of age data and scales it so that it can be plotted
#' in the same plot of a histogram of the age data
#'
#' @param dat data.frame
#' @param binwidth Histogram binwidth
#' @param bw Density bandwidth
#' @param type 'kde': KDE; 'pdd': detrital zircon PDD
#' @param age_range Age range to calculated density over
#'
#' @return Returns density
#' @export
calc_dens_hist <- function(dat, binwidth=50, bw=30, type='kde',
                           age_range=c(0, 4560)) {
  dens <- calc_dens(dat, bw, type, age_range)
  max_y <- max(dens$y)
  # binmin <- find_plot_min(dat$age, accuracy=binwidth)
  # binmax <- find_plot_max(dat$age, accuracy=binwidth)
  # breaks <- seq(binmin, binmax, binwidth)
  hist_data <- graphics::hist(dat$age, breaks=seq(0, 4560, binwidth),
                              plot=FALSE, right=TRUE)
  dens$y <- dens$y * (max(hist_data$counts) / max_y)
  return(dens)
}

#' Find minimum value for plotting
#'
#' Find the minimum value for histogram plotting.
#'
#' @param x vector of values
#' @param accuracy round to nearest
#' @export
find_plot_min <- function(x, accuracy=100) {
  minimum <- floor(min(x) / accuracy) * accuracy
}

#' Find maximum value for plotting.
#'
#' Find the maximum value for histogram plotting.
#'
#' @param x vector of values
#' @param accuracy round to nearest
#' @export
find_plot_max <- function(x, accuracy=100) {
  maximum <- ceiling(max(x) / accuracy) * accuracy
}

#' Wrapper function for \code{find_plot_min} and \code{find_plot_max}
#'
#' Find the minimum and maximum values for histogram plotting.
#'
#' @param x Age data
#' @param accuracy Round to nearest
#'
#' @return Returns vector of minimum and maximum plotting values
#' @export
find_plot_min_max <- function(x, accuracy=100) {
  return(c(find_plot_min(x, accuracy), find_plot_max(x, accuracy)))
}

#' Calculate 1d likeness of detrital zircon populations
#'
#' Calculates the likeness of detrital zircon populations in 1 dimension after
#' Satkoski et al. (2013).
#'
#' @param x vector
#' @param y vector
#' @param bw bandwidth
#' @param digits number, round result to significant digits
#'
#' @references Satkoski, A.M., Wilkinson, B.H., Hietpas, J., Samson, S.D., 2013.
#' Likeness among detrital zircon populations - An approach to the comparison of
#' age frequency data in time and space. GSA Bulletin 125, 1783-1799.
#' @export
satkoski_1d <- function(x, y, bw=30, digits=3) {

  a <- calc_dens(x, bw=bw)
  b <- calc_dens(y, bw=bw)

  L <- 1 - (sum(abs(a$y - b$y)) / 2)
  round(L, digits)
}

#' Calculate 2d (age and Lu-Hf) likeness of detrital zircon populations
#'
#' Calculates the likeness of detrital zircon populations in 2 dimensions after
#' Satoski et al. (2013).
#'
#' @param x vector
#' @param y vector
#' @param bw vector of density bandwidths
#' @param digits number, round result to significant digits
#'
#' @references Satkoski, A.M., Wilkinson, B.H., Hietpas, J., Samson, S.D., 2013.
#' Likeness among detrital zircon populations - An approach to the comparison of
#' age frequency data in time and space. GSA Bulletin 125, 1783-1799.
#' @export
satkoski_2d <- function(x, y, bw=c(30, 2.5), digits=3) {
  if (all(is.na(x$ehf_i)) | all(is.na(y$ehf_i))) {
    return(NA)
  } else {
    n <- 100
    bw <- bw * 4
    lims <- c(0, 4560, -30, 30)
    x <- x[!is.na(x$ehf_i), ]
    y <- y[!is.na(y$ehf_i), ]
    a <- MASS::kde2d(x=x$age, y=x$ehf_i, h=bw, n=n, lims=lims)$z
    b <- MASS::kde2d(x=y$age, y=y$ehf_i, h=bw, n=n, lims=lims)$z
    a <- a / sum(a)
    b <- b / sum(b)
    L <- 1 - (sum(abs(a - b)) / 2)
    round(L, digits)
  }
}

#' Pairwise Satkoski likeness
#'
#' Populate a matrix with pairwise Satkoski 1d likeness.
#'
#'@param dat data.frame
#'@param bw density bandwidth
#'@param digits number, round result to significant digits
#' @references Satkoski, A.M., Wilkinson, B.H., Hietpas, J., Samson, S.D., 2013.
#' Likeness among detrital zircon populations - An approach to the comparison of
#' age frequency data in time and space. GSA Bulletin 125, 1783-1799.
#' @export
satkoski_1d_matrix <- function(dat, bw=30, digits=3) {
  populate_matrix(dat, FUN=satkoski_1d, bw=bw, digits=3)
}

#' Pairwise 2d Satkoski likeness
#'
#' Populate a matrix with pairwise Satkoski 12 likeness.
#'
#'@param dat data.frame
#'@param bw vector of density bandwidths
#'@param digits number, round result to significant digits
#'
#' @references Satkoski, A.M., Wilkinson, B.H., Hietpas, J., Samson, S.D., 2013.
#' Likeness among detrital zircon populations - An approach to the comparison of
#' age frequency data in time and space. GSA Bulletin 125, 1783-1799.
#' @export
satkoski_2d_matrix <- function(dat, bw=c(30, 2.5), digits=3) {
  populate_matrix(dat, FUN=satkoski_2d, bw=bw, digits=digits)
}

#' Populate matrix
#'
#' @param dat data.frame
#' @param FUN Function used to populate matrix
#' @param ... Additional parameters passed to function
#'
#' @return Populated matrix
#' @export
populate_matrix <- function(dat, FUN, ...) {
  n <- length(unique(dat$sample))
  if (n < 2) stop('Select more samples')
  name <- as.character(unique(dat$sample))
  len <- length(name)
  mat <- matrix(nrow=n, ncol=n)
  colnames(mat) <- name
  rownames(mat) <- name
  for (i in 1:(len - 1)) {
    for (j in 2:len) {
      if (!(i == j) & ((is.na(mat[i, j])) | is.na(mat[j, i]))) {
        L <- FUN(dat[dat$sample == name[i], ],
                         dat[dat$sample == name[j], ], ...)
        mat[i, j] <- L
        mat[j, i] <- L
      }
    }
  }
  return(mat)
}

#' Calculate hafnium values.
#'
#' Calculates the initial 176Hf/177Hf values, the initial epsilon hafnium
#' values, the model age using the measured 176Lu/177Hf value and
#' the model age assuming the parental magma was produced from an average continental crust
#' (176Lu/177Hf = 0.015) that originally was derived from the depleted mantle
#' (Griffin, 2004).
#'
#' @param dat data.frame, list or matrix of hafnium values
#' @param constants vector of constants which must be in the order
#' decay constant 176Lu, 176/177Hf CHUR, 176Lu/177Hf CHUR, 176/177Hf DM,
#' 176Lu/177Hf DM and 176Lu/177Hf value used for two-stage depleted mantle
#' model age calculations
#' @export
#' @references Bouvier, A., Vervoort, J.D. & Patchett, P.J. 2008.
#' The Lu-Hf and Sm-Nd isotopic composition of CHUR:
#' Constraints from unequilibrated chondrites and implications for the bulk composition of terrestrial planets.
#' Earth And Planetary Science Letters 273(1-2), 48-57.
#' @references Griffin, W., Belousova, E., Shee, S., Pearson, N. and O'Reilly, S. 2004.
#' Archean crustal evolution in the northern Yilgam Craton:
#' U-Pb and Hf-isotope evidence from detrital zircons. Precambrian Research, 231-282.
#' @references Soderlund, U., Patchett, J., Vervoort, J. & Isachsen, C. 2004.
#' The Lu-176 decay constant determined by Lu-Hf and U-Pb isotope systematics of Precambrian mafic intrusions.
#' Earth And Planetary Science Letters 219(3-4), 311-324.
calc_hf <- function(dat, constants) {
  lambda_lu <- constants[1]
  hfhf_chur <- constants[2]
  luhf_chur <- constants[3]
  hfhf_dm <- constants[4]
  luhf_dm <- constants[5]
  luhf_zrc <- constants[6]
  if ('ehf_i' %in% names(dat)) {
    dat <- dat[, !(names(dat) %in% c('hfhf', 'hfhfse', 'luhf'))]
  }
  if (is.vector(dat) || ncol(dat) < 3) {
    stop('Data must contain at least three columns')
  }
  hf_chur <- hfhf_chur - (luhf_chur * (exp(lambda_lu * (dat$age * (10^6))) - 1))
  if ('ehf_i' %in% names(dat)) {
    hf_i <- hf_chur * ((dat$ehf_i / 10^4) + 1)
  } else {
    if ('hfhf' %in% names(dat)) {
      hf_i <- dat$hfhf - (dat$luhf * (exp(lambda_lu * (dat$age * (10^6))) - 1))
      ehf_i <- ((hf_i / hf_chur) - 1) * 10^4
    }
  }
  if ('hfhfse' %in% names(dat)) {
    hf_2se <- 2 * dat$hfhfse
    ehf_2se <- ((((hf_i + hf_2se) / (hfhf_chur - (luhf_chur * (exp(lambda_lu *
                                                 (dat$age * (10^6))) - 1)))) -
                                                  1)*10^4) - ehf_i
  } else {
    hf_2se <- rep(NA, nrow(dat))
    ehf_2se <- hf_2se
  }
  hf_dm <- hf_i + (luhf_zrc * (exp(lambda_lu * (dat$age * (10^6))) -1))
  t_dm2 <- ((1 / (lambda_lu)) *
              log(((hf_dm - hfhf_dm) / (luhf_zrc - luhf_dm)) + 1)) / 10^6
  if ('ehf_i' %in% names(dat)) {
    dat <- cbind(dat, ehf_2se, hf_i, hf_2se, hf_chur, t_dm2)
  } else {
    if ('hfhf' %in% names(dat)) {
      t_dm <- (((1 / (lambda_lu)) *
                  log(((dat$hfhf - hfhf_dm) / (dat$luhf - luhf_dm)) + 1)) / 10^6)
      dat <- cbind(dat, hf_i, hf_2se, ehf_i, ehf_2se, t_dm, t_dm2)
    }
  }
}

#' Dvoretzky-Kiefer-Wolfowitz inequality
#'
#' Calculate confidence bands for ecdfs using the Dvoretzky-Kiefer-Wolfowitz
#' inequality.
#'
#' @param dat data.frame
#' @param column which column to use
#' @param alpha Desired alpha level
#'
#' @return data.frame with ecdf and confidence bands
#' @export
#'
#' @references Dvoretzky, A., Kiefer, J., Wolfowitz, J., 1956. Asymptotic
#' Minimax Character of the Sample Distribution Function and of the Classical
#' Multinomial Estimator. Ann. Math. Stat. 27, 642-669.
#' doi:10.1214/aoms/1177728174
calc_dkw <- function(dat, column='age', alpha=0.05) {
  if ('sample' %in% names(dat)) {
    sample_name <- dat$sample[1]
  } else {
    sample_name <- NA
  }
  x <- dat[, column]
  x <- x[!is.na(x)]
  y <- stats::ecdf(x)(sort(x))
  n <- length(x)
  x <- c(0, sort(x), 4560)
  y <- c(0, y, 1)
  epsilon <- sqrt(log(2 / alpha) / (2 * n))
  low <- pmax(y - epsilon, 0)
  high <- pmin(y + epsilon, 1)
  sample <- rep(sample_name, length(x))
  data.frame(x=x, y=y, low=low, high=high, sample=sample)
}

#' Calculate 1-O
#'
#' @param dat1 data.frame
#' @param dat2 data.frame
#' @param column string of name of column to use ('age' or 't_dm2')
#' @param alpha alpha level
#' @param digits number of digits
#'
#' @return 1-O
#' @export
#' @references Andersen, T., Elburg, M., Cawthorn-Blazeby, A., 2015.
#' U-Pb and Lu-Hf zircon data in young sediments reflect sedimentary recycling
#' in eastern South Africa. J. Geol. Soc. London. 2006-2015.
#' doi:10.1144/jgs2015-006
#'
calc_o_param <- function(dat1, dat2, column, alpha=0.05, digits=2) {
  x <- dat1[, column]
  y <- dat2[, column]
  if (all(is.na(x)) | all(is.na(y))) {
    return(NA)
  }
  else {
    epsilon_one <- sqrt(log(2 / alpha) / (2 * length(x)))
    epsilon_two <- sqrt(log(2 / alpha) / (2 * length(y)))
    x_sort <- sort(x)
    x_y <- stats::ecdf(x_sort)(x_sort)
    y_sort <- sort(y)
    y_y <- stats::ecdf(y_sort)(y_sort)
    x_sort <- c(0, x_sort, 4560)
    x_y <- c(0, x_y, 1)
    y_sort <- c(0, y_sort, 4560)
    y_y <- c(0, y_y, 1)
    x_out <- seq(0, 4560)
    interpolated_one <- stats::approx(x_sort, x_y, xout=x_out, ties='ordered')$y
    interpolated_two <- stats::approx(y_sort, y_y, xout=x_out, ties='ordered')$y
    interpolated_one_low <- pmax(interpolated_one - epsilon_one, 0)
    interpolated_one_up <- pmin(interpolated_one + epsilon_one, 1)
    interpolated_two_low <- pmax(interpolated_two - epsilon_two, 0)
    interpolated_two_up <- pmin(interpolated_two + epsilon_two, 1)
    up <- ifelse((((interpolated_two_up <= interpolated_one_up) &
                     (interpolated_two_up >= interpolated_one_low)) |
                    ((interpolated_one_up <= interpolated_two_up) &
                       (interpolated_one_up >= interpolated_two_low))), 1, 0)
    low <- ifelse((((interpolated_two_low >= interpolated_one_low) &
                      (interpolated_two_low <= interpolated_one_up)) |
                     ((interpolated_one_low >= interpolated_two_low) &
                        (interpolated_one_low <= interpolated_two_up))), 1, 0)
    O <- (length(up[up == 1]) + length(low[low == 1])) / (2 * length(x_out))
    round(1 - O, digits)
  }
  # delta <- epsilon_one + epsilon_two
  # absolute <- pmax((abs(interpolated_one - interpolated_two) - delta), 0)
  # absolute <- absolute[absolute != 0]
  # round(1- (1 - length(absolute) / length(x_out)), digits)
}

#' Populate matrix with age 1-O
#'
#' @param dat data.frame
#' @param alpha alpha level
#' @param digits number of digits
#'
#' @return matrix of 1-O for ages
#' @export
#' @references Andersen, T., Elburg, M., Cawthorn-Blazeby, A., 2015.
#' U-Pb and LuHf zircon data in young sediments reflect sedimentary recycling
#' in eastern South Africa. J. Geol. Soc. London. 2006-2015.
#' doi:10.1144/jgs2015-006
o_param_matrix_age <- function(dat, alpha=0.05, digits=2) {
  populate_matrix(dat, FUN=calc_o_param, column='age', alpha=alpha,
                  digits=digits)
}

#' Populate matrix with model age 1-O
#'
#' @param dat data.frame
#' @param alpha alpha level
#' @param digits number of digits
#'
#' @return matrix of 1-O for model ages
#' @export
#' @references Andersen, T., Elburg, M., Cawthorn-Blazeby, A., 2015.
#' U-Pb and Lu-Hf zircon data in young sediments reflect sedimentary recycling
#' in eastern South Africa. J. Geol. Soc. London. 2006-2015.
#' doi:10.1144/jgs2015-006
o_param_matrix_tdm <- function(dat, alpha=0.05, digits=2) {
  populate_matrix(dat, FUN=calc_o_param, column='t_dm2', alpha=alpha,
                  digits=digits)
}

#' Combine two square matrices
#'
#' @param mat1 Matrix for upper triangle
#' @param mat2 Matrix for lower triangle
#'
#' @export
#'
combine_matrices <- function(mat1, mat2) {
  mat1[lower.tri(mat1)] <- NA
  mat1[is.na(mat1)] <- mat2[is.na(mat1)]
  mat1
}

#' Produce CHUR and DM lines
#'
#' Calculate CHUR and DM lines used for epsilon-Hf vs. age and 176/177Hf vs. age
#' plots.
#'
#' @param range range over which to calculate lines
#' @param plot_type 'ehf' = epsilon-Hf; any thing else gives 176/177Hf
#' @param constants vector of constants which must be in the order
#' decay constant 176Lu, 176/177Hf CHUR, 176Lu/177Hf CHUR, 176/177Hf DM and
#' 176Lu/177Hf DM
#'
#' @export
#'
#' @references Griffin, W., Pearson, N., Belousova, E., Jackson, S., van
#' Achterbergh, E., O'Reilly, S. and Shee, S. 2000. The Hf isotope composition
#' of cratonic mantle:
#' LAM-MC-ICPMS analysis of zircon megacrysts in kimberlites.
#' Geochimica et Cosmochimica Acta 64(1), 133-147.
#' @references Soderlund, U., Jonathan Patchett, P., Vervoort, J.D. and Isachsen
#' C.E. 2004. The 176Lu decay constant determined by Lu-Hf and U-Pb isotope
#' systematics of Precambrian mafic intrusions. Earth and Planetary Science
#' Letters 219, 311-324.
#' @references Bouvier, A., Vervoort, J.D. and Jonathan Patchett P. 2008. The
#' Lu-Hf and Sm-Nd isotopic composition of CHUR: Constraints from unequilibrated
#' chondrites and implications for the bulk composition of terrestrial planets.
#' Earth and Planetary Science Letters 273, 48-57.
hf_lines <- function(range=c(0, 4560), plot_type='ehf', constants) {
  lambda_lu <- constants[1]
  hfhf_chur <- constants[2]
  luhf_chur <- constants[3]
  hfhf_dm <- constants[4]
  luhf_dm <- constants[5]
  x <- seq(range[1], range[2], by=1)
  length_x <- length(x)
  #CHUR (Bouvier,2008)
  chur <- hfhf_chur - luhf_chur * ((exp(lambda_lu * x * 10^6)) - 1)
  #DM (Griffin,2000)
  dm <- hfhf_dm - luhf_dm *((exp(lambda_lu * x * 10^6)) - 1)
  if (plot_type == 'ehf') {
    dm <- ((dm / chur) - 1) * 10^4
    chur <- rep(0, length_x)
  }
  type_chur <- rep('CHUR', length_x)
  type_dm <- rep('DM', length_x)
  type <- c(type_chur, type_dm)
  x <- rep(x, 2)
  line <- c(chur, dm)
  linedata <- data.frame(x=x, line=line, type=type)
}


#' Calculate quantiles
#'
#' Split up data.frame by sample-column and calculate quantiles
#'
#' @param dat data.frame
#' @param column which column in data.frame to use
#' @param alpha alpha-level (not yet used)
#' @param type type of quantile calculation (passed on to stats::quantile)
#'
#' @export
#'
calc_quantiles <- function(dat, column='t_dm2', alpha=0.05, type=8) {
  x <- dat[, column]
  quantiles <- stats::aggregate(x=x, by=list(dat$sample),
                                FUN=stats::quantile, type=type, na.rm=TRUE)
  x <- as.data.frame(quantiles$x)
  x$iqr <- x$`75%` - x$`25%`
  x$sample <- quantiles$Group.1
  names(x) <- c('zero', 'twentyfive', 'fifty', 'seventyfive', 'onehundred',
                'iqr', 'sample')
  return(x)
}

#' Calculate confidence bands for lower and upper quartile
#'
#' @param dat data.frame
#' @param column column to use for calculations
#' @param alpha alpha-level
#'
#' @export
#'
quant_bounds <- function(dat, column='t_dm2', alpha=0.05) {
  column <- dat[, column]
  sample <- dat$sample[1]
  if (all(is.na(column))) {
    data.frame(x=NA, y=NA, ymin=NA, ymax=NA, xmin=NA, xmax=NA, sample=sample)
  } else {
    sort_column <- sort(column)
    y <- stats::ecdf(column)(sort(column))
    epsilon <- sqrt(log(2 / alpha) / (2 * length(column)))
    low <- pmax(y - epsilon, 0)
    high <- pmin(y + epsilon, 1)
    sort_age_low <- data.frame(x=sort(column), y=low)
    sort_age_high <- data.frame(x=sort(column), y=high)
    ll <- stats::approx(x=sort_age_low$y, y=sort_age_low$x, xout=c(0.25),
                        ties='ordered')$y
    lu <- stats::approx(x=sort_age_low$y, y=sort_age_low$x, xout=c(0.75),
                        ties='ordered')$y
    if(is.na(lu)) lu <- 4560
    ul <- stats::approx(x=sort_age_high$y, y=sort_age_high$x, xout=c(0.25),
                        ties='ordered')$y
    if(is.na(ul)) ul <- 0
    uu <- stats::approx(x=sort_age_high$y, y=sort_age_high$x, xout=c(0.75),
                        ties='ordered')$y
    lq_dist <- stats::quantile(column, probs=c(0.25), type=8, na.rm=TRUE)
    uq_dist <- stats::quantile(column, probs=c(0.75), type=8, na.rm=TRUE)
    data.frame(x=lq_dist, y=uq_dist, ymin=uu, ymax=lu, xmin=ul,  xmax=ll, sample)
  }
}

#' Calculate mixing model
#'
#' Gaussian mixing model for detrital zircon data, using lower quantile upper
#' quantile plot
#'
#' @param mu1 first mean
#' @param sig1 first standard deviation
#' @param mu2 second mean
#' @param sig2 second standard deviation
#'
#' @examples
#' dzr_mix(500, 50, 1000, 100)
#' @export
#'
dzr_mix <- function(mu1, sig1, mu2, sig2) {
  X <- rep(NA, 100)
  S <- rep(NA, 100)
  lq <- matrix(nrow=11, ncol=100)
  uq <- lq
  for (j in seq(1, 100, 1)) {
    for (i in seq(1, 11, 1)) {
      X[1: (100 - 10 * (i - 1))] <- mu1
      X[(101 - 10 * (i - 1)): 100] <- mu2
      S[1: (100 - 10 * (i - 1))] <- sig1
      S[(101 - 10 * (i - 1)): 100] <- sig2

      x <- stats::rnorm(100, mean=X, sd=S)
      y <- stats::ecdf(x)(x)
      lq[i, j] <- stats::quantile(x, probs=0.25, type=8)
      uq[i, j] <- stats::quantile(x, probs=0.75, type=8)
    }
  }
  data.frame(lq=rowMeans(lq), uq=rowMeans(uq))
}

#' Ready 1-O matrix for tile plot
#'
#' @param x 1-O parameter vector
#'
#' @export
tile_func <- function(x) {
  if (is.na(x)) {
    return(NA)
  }
  if (x >= 0.05) {
    return(2)
  }
  if (x > 0 & x < 0.05) {
    return(1)
  } else {
    return(x)
  }
}

#' Apply tile_func to vector
#'
#' @param z 1-O parameter vector
#'
#' @export
#'
tiling <- function(z) {
  sapply(z, tile_func)
}

#' Produce data.frame of 1-O matrix suitable for geom_tile
#'
#' @param dat data.frame
#' @param type What to calculate
#'
#' @export
#'
make_tiling <- function(dat, type) {
  if (type == 'age') {
    mat <- o_param_matrix_age(dat$age)
    mat <- mat[, rev(seq_len(ncol(mat)))]
    tile_mat <- data.frame(x=rownames(mat)[row(mat)], y=colnames(mat)[col(mat)],
                           z=as.factor(tiling(c(mat))))
    tile_mat$x <- factor(tile_mat$x, levels=rownames(mat))
    tile_mat$y <- factor(tile_mat$y, levels=colnames(mat))
  }
  if (type == 'tdm') {
    mat <- o_param_matrix_tdm(dat$hf)
    mat <- mat[, rev(seq_len(ncol(mat)))]
    tile_mat <- data.frame(x=rownames(mat)[row(mat)], y=colnames(mat)[col(mat)],
                           z=as.factor(tiling(c(mat))))
    tile_mat$x <- factor(tile_mat$x, levels=rownames(mat))
    tile_mat$y <- factor(tile_mat$y, levels=colnames(mat))
  }
  if (type == 'combine') {
    mat_age <- o_param_matrix_age(dat$age)
    mat_tdm <- o_param_matrix_tdm(dat$hf)
    mat_age[lower.tri(mat_age)] <- NA
    mat <- mat_age
    mat[is.na(mat)] <- mat_tdm[is.na(mat)]
    mat <- t(mat)
    mat <- mat[, rev(seq_len(ncol(mat)))]
    tile_mat <- data.frame(x=rownames(mat)[row(mat)], y=colnames(mat)[col(mat)],
                           z=as.factor(tiling(c(mat))))
    tile_mat$x <- factor(tile_mat$x, levels=rownames(mat))
    tile_mat$y <- factor(tile_mat$y, levels=colnames(mat))
  }
  tile_mat
}

#' Calculate upper and lower concordia intercepts from discordant detrital
#' zircon data
#'
#' @param dat data.frame
#' @param step Chord spacing
#'
#' @export
#'
#' @references Reimink, J.R., Davies, J.H.F.L., Waldron, J.W.F., Rojas, X.
#' (2016). Dealing with discordance: a novel approach for analysing U-Pb
#' detrital zircon datasets. Journal of the Geological Society.
#' doi: 10.1144/jgs2015-114
#'
reimink <- function(dat, step=5) {
  t1 <- seq(0, 4500 - step, step)
  t2 <- seq(step, 4500, step)
  grid <- expand.grid(t1, t2)
  grid <- grid[grid$Var2 > grid$Var1,]
  dat$sigma75 <- stats::median(dat$sigma75)
  dat$sigma68 <- stats::median(dat$sigma68)
  result <- calc_p_apply(dat, grid$Var2, grid$Var1)
  lower <- stats::aggregate(result$p_disc, by=list(result$t1), max)
  upper <- stats::aggregate(result$p_disc, by=list(result$t2), max)
  names(lower) <- c('x', 'y')
  names(upper) <- c('x', 'y')
  lower$x <- as.numeric(as.character(lower$x))
  upper$x <- as.numeric(as.character(upper$x))
  lower$type <- rep('lower', nrow(lower))
  upper$type <- rep('upper', nrow(upper))
  out <- rbind(lower, upper)
}


#' Calculate intercepts and associated p-value
#'
#' @param dat data.frame
#' @param t2 upper intercept age
#' @param t1 lower intercept age
#'
#' @export
#'
calc_p_apply <- function(dat, t2, t1) {
  lambda238 <- 1.55125 * (10^-10)
  X <- dat$r75
  Y <- dat$r68
  sigma75 <- dat$sigma75
  sigma68 <- dat$sigma68
  rho <- dat$rho
  age <- dat$age * 10^6
  disc <- 1 - (Y / ((exp(lambda238 * age)-1)))
  ab <- calc_ab(t2, t1)
  slope <- ab$a
  yintercept <- ab$b
  n <- nrow(dat)
  p_disc <- rep(NA, length(slope))
  for (i in seq_len(length(slope))) {
    S <- atan2((2 * rho * sigma75 * sigma68) / (sigma75^2 - sigma68^2), 2)
    a <- ((slope[i] * cos(S)) - sin(S)) / (cos(S) + (slope[i] * sin(S)))
    b <- (yintercept[i] + (slope[i] * X) - Y) / (cos(S) + (slope[i] * sin(S)))
    B <- (sigma75 / sigma68) * a
    A <- b / sigma68
    p <- (1 / (2 * pi * sigma75 * sigma68)) * exp(-0.5 * (A^2 / (1 + B^2)))
    p <- p / n
    p_disc[i] <- sum(p * disc)
  }
  return(data.frame(t1, t2, p_disc))
}


#' Calculate slope and intercept
#'
#' @param t2 upper intercept
#' @param t1 lower intercept
#'
#' @export
#'
calc_ab <- function (t2, t1) {
  y2 <- concY(t2)
  y1 <- concY(t1)
  x2 <- concX(t2)
  x1 <- concX(t1)
  a = (y2 - y1) / (x2 - x1)
  b = y2 - a * x2
  ab <- data.frame(a=a, b=b)
}

#' Calculate U235 at given age
#'
#' @param age input age
#'
#' @export
#'
concX <- function(age) {
  age <- age * 10^6
  lambda235 <- 9.8485 * (10^-10)
  cx <- exp(lambda235*age) - 1
  return(cx)

}

#' Calculate U238 at given age
#'
#' @param age input age
#'
#' @export
#'
concY <- function(age) {
  age <- age * 10^6
  lambda238 <- 1.55125 * (10^-10)
  cy <- exp(lambda238*age) - 1
  return(cy)
}

#' Find maxima.
#'
#'
#' @param dist distribution.
#' @param xmin minimum value of distribution.
#' @param inc increment.
#'
#' @export
find_maxima <- function(dist, xmin, inc) {
  difference <- c(0, diff(dist))
  difference2 <- c(difference[2:length(difference)], 0)
  maximadata <- ((which(difference > 0 & difference2 <= 0) * inc) + xmin - 1)
}

---
title: "detzrcr user guide"
author: "Magnus Kristoffersen"
date: "`r Sys.Date()`"
output:
    rmarkdown::html_document:
      fig_width: 2
      fig_height: 2
      fig_caption: true
vignette: >
  %\VignetteIndexEntry{detzrcr user guide}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---
## Installing and running the package

The latest version of [R](https://www.r-project.org/) must be installed and
[Rstudio](https://posit.co/) is recommended.
The released version of the package can be installed from
[CRAN](https://CRAN.R-project.org/package=detzrcr)

```{r, eval=FALSE}
install.packages("detzrcr")
```
or the development version can be installed from 
[github](https://github.com/magnuskristoffersen/detzrcr) (requires `devtools`)

```{r, eval=FALSE}
# install.packages("devtools")
devtools::install_github(magnuskristoffersen/detzrcr)
```

Launch the user interface with
```{r, eval=FALSE}
library("detzrcr")
run_detzrcr()
```

## Required input format

The data must be input in a csv-file which is uploaded to the `Data input` tab
(Fig. 1) in the *shiny* interface. 

![Fig. 1: Data input tab](data_input.png)

This file has certain requirements

* Each row contains one analysis
* Contains columns with headers **sample** and **age**
* Discordancy and uncertainty of the age data, if included, in columns with
headers **disc** and **uncert**, respectively
* Discordancy is percentage discordance where 0 signifies concordance, a
negative value normal and a positive value reverse discordance
* Uncertainty is the $1\sigma$ absolute uncertainty of the age
* Hf data as measured $^{176}$Hf/$^{177}$Hf and measured $^{176}$Lu/$^{177}$Hf
in columns **hfhf** and **luhf**, respectively or as initial $\epsilon$Hf in
column **ehf_i**
* Data in Hf columns can be left empty
* It is recommended to include an identifier for the analysis, there is no
required name for this column but it can be **point**, **analysis** etc.

```{r, echo=FALSE, results='asis'}
dat <- read.csv('karoo.csv')
knitr::kable(head(dat, 5), format='pandoc', caption='Example input format')
```

## Features

* Plot age data as KDEs and PDPs
* Plot age and Hf model age as ECDFs
    + Add confidence bands
* Plot age and Hf model age in upper quantile vs. lower quantile space
    + Add confidence limits
    + Add Gaussian mixture model
* Plot Hf data as initial $\epsilon$Hf vs. age or initial $^{176}$Hf/$^{177}$Hf
vs. age
    + Add sample contours
* Export publication quality figures (as pdf)
* Calculate similarity measures
    + Satkoski likeness (Satkoski et al. 2013)
    + 1-O (Andersen et al. 2016a)
* Export similarity and Hf tables as csv files

## Usage
`detzrcr` has been designed to be easy to use with no prior knowledge of
`R` required. We therefore chose to build the package as a `shiny` 
based user interface where the user can simply upload a data file and output
publication quality figures (using `ggplot2`) and tables.

Data is uploaded in the `Data input` tab (Fig. \ref{fig1}). From this tab a
concordance filter can be applied to the data (defaults to $\pm$ 10%). The
different featuresof the R-package can then be used by selecting the appropriate
panel tab in the user interface. By selecting the `Density distribution` tab the
user can plot the U-Pb age data as KDEs (user selectable fixed bandwidth) or
PDPs (local variable bandwidth based on the absolute $1\sigma$ uncertainty).
Which samples to plot and in what order can be selected from the `Select sample`
menu (Fig. fig2). This feature is also available for all other plot and table
options.

![Fig. 2: Density distribution tab](kde.png)

ECDFs of U-Pb age data as well as Hf model age data can be plotted from the
`ECDF` tab, 95% confidence bands calculated using the Dvoretzky-Kiefer-Wolfowitz
(DKW) inequality (Dvoretzky et al. 1956) can be added.
In the `UQ vs. LQ` tab U-Pb age data and Hf model age data is plotted in UQ vs.
LQ space where (95% DWK) confidence limits and a gaussian mixture model can be
added (Fig. 3).

![Fig. 3: UQ vs. LQ with mixture model](uqlq.png)

Hf data can be plotted from the `Lu-Hf` tab as initial $\epsilon$Hf vs. age or
initial $^{176}$Hf/$^{177}$Hf vs. age, where sample contours can be added.
The implemented similarity measures are found in the `Likeness` and `1-O` tabs
where the likeness (Satkoski et al. 2013) and 1-O (Andersen et al. 2016a)
parameters are calculated and displayed. Likeness can be calculated in both 1
dimension (U-Pb age only) and 2 dimensions (U-Pb age and initial $\epsilon$Hf).
1-O can be calculated for U-Pb age and Hf model age. In each of these panels the
two options can be combined into one table with U-Pb data in the upper triangle
and Hf data in the lower triangle (Fig. 4). All tables can be saved to
csv-files.

![Fig. 4: 1-O table](1-o.png)

The 1-O table can also be displayed graphically (Fig. 5) where red indicates
$1-O \geq 0.05$, white $0 < 1-O < 0.05$ and green $1-O = 0$.

![Fig. 5: 1-O table as figure](oplot.png)

Constants used in Hf calculations can be changed in the `Constants` tab.
Default values are as follows

* $\lambda ^{176}Lu = 1.867 \times 10^{-11} a^{-1}$ (Söderlund et al. 2004) 
* CHUR values: $^{176}$Lu/$^{177}$Hf$=0.0336$, $^{176}$Hf/$^{177}$Hf$=0.282785$
(Bouvier et al. 2008)
* Depleted mantle values: $^{176}$Lu/$^{177}$Hf$=0.0388$,
$^{176}$Hf/$^{177}$Hf$=0.28325$ (from Griffin et al. (2004) modified to 
the aforementioned decay constant and CHUR values)
* $^{176}$Lu/$^{177}$Hf value used in two step depleted model age $=0.015$

## Reimink intercepts

The method of Reimink et al. (2016) to find the most likely time of lead loss in
suites of discordant detrital zircon is added to the package. To use this
feature the input data must include $^{207}Pb/^{235}U$, $1\sigma$ absolute
error of $^{207}Pb/^{235}U$, $^{206}Pb/^{238}U$, $1\sigma$ absolute error of
$^{206}Pb/^{238}U$, and error correlation with the column names r75, sigma75,
r68, sigma68 and rho, respectively.

### Data source
All data shown herein are taken from Andersen et al. (2016b).

### References
Andersen, T., Elburg, M., Cawthorn-Blazeby, A., 2016a. U–Pb and Lu–Hf zircon data
in young sediments reflect sedimentary recycling in eastern South Africa. J.
Geol. Soc. London. 173, 337–351. https://dx.doi.org/10.1144/jgs2015-006

Andersen, T., Kristoffersen, M., Elburg, M.A., 2016b. How far can we trust
provenance and crustal evolution information from detrital zircons? A south
African case study. Gondwana Research 34, 129–148.
https://doi.org/10.1016/j.gr.2016.03.003

Bouvier, A., Vervoort, J.D., Patchett, P.J., 2008. The Lu-Hf and Sm-Nd isotopic
composition of CHUR: Constraints from unequilibrated chondrites and implications
for the bulk composition of terrestrial planets. Earth Planet. Sci. Lett. 273,
48–57. https://dx.doi.org/10.1016/j.epsl.2008.06.010

Dvoretzky, A., Kiefer, J., Wolfowitz, J., 1956. Asymptotic Minimax Character of
the Sample Distribution Function and of the Classical Multinomial Estimator.
Ann. Math. Stat. 27, 642–669. https://dx.doi.org/10.1214/aoms/1177728174

Griffin, W.L., Belousova, E.A., Shee, S.R., Pearson, N.J., O’Reilly, S.Y., 2004.
Archean crustal evolution in the northern Yilgam Craton: U-Pb and Hf-isotope
evidence from detrital zircons. Precambrian Res. 131, 231–282.

Reimink, J. R., Davies, J. H. F. L., Waldron, J. W. F., & Rojas, X. (2016).
Dealing with discordance: a novel approach for analysing U-Pb detrital zircon
datasets. Journal of the Geological Society, 2015-114.
https://doi.org/10.1144/jgs2015-114

Satkoski, A.M., Wilkinson, B.H., Hietpas, J., Samson, S.D., 2013. Likeness among
detrital zircon populations — An approach to the comparison of age frequency
data in time and space. GSA Bull. 125, 1783–1799.
https://dx.doi.org/10.1130/B30888.1

Söderlund, U., Patchett, P.J., Vervoort, J.D., Isachsen, C.E., 2004. The 176Lu
decay constant determined by Lu-Hf and U-Pb isotope systematics of Precambrian
mafic intrusions. Earth Planet. Sci. Lett. 219, 311–324.
https://dx.doi.org/10.1016/S0012-821X(04)00012-3

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{tile_func}
\alias{tile_func}
\title{Ready 1-O matrix for tile plot}
\usage{
tile_func(x)
}
\arguments{
\item{x}{1-O parameter vector}
}
\description{
Ready 1-O matrix for tile plot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{calc_ab}
\alias{calc_ab}
\title{Calculate slope and intercept}
\usage{
calc_ab(t2, t1)
}
\arguments{
\item{t2}{upper intercept}

\item{t1}{lower intercept}
}
\description{
Calculate slope and intercept
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{calc_p_apply}
\alias{calc_p_apply}
\title{Calculate intercepts and associated p-value}
\usage{
calc_p_apply(dat, t2, t1)
}
\arguments{
\item{dat}{data.frame}

\item{t2}{upper intercept age}

\item{t1}{lower intercept age}
}
\description{
Calculate intercepts and associated p-value
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{calc_hf}
\alias{calc_hf}
\title{Calculate hafnium values.}
\usage{
calc_hf(dat, constants)
}
\arguments{
\item{dat}{data.frame, list or matrix of hafnium values}

\item{constants}{vector of constants which must be in the order
decay constant 176Lu, 176/177Hf CHUR, 176Lu/177Hf CHUR, 176/177Hf DM,
176Lu/177Hf DM and 176Lu/177Hf value used for two-stage depleted mantle
model age calculations}
}
\description{
Calculates the initial 176Hf/177Hf values, the initial epsilon hafnium
values, the model age using the measured 176Lu/177Hf value and
the model age assuming the parental magma was produced from an average continental crust
(176Lu/177Hf = 0.015) that originally was derived from the depleted mantle
(Griffin, 2004).
}
\references{
Bouvier, A., Vervoort, J.D. & Patchett, P.J. 2008.
The Lu-Hf and Sm-Nd isotopic composition of CHUR:
Constraints from unequilibrated chondrites and implications for the bulk composition of terrestrial planets.
Earth And Planetary Science Letters 273(1-2), 48-57.

Griffin, W., Belousova, E., Shee, S., Pearson, N. and O'Reilly, S. 2004.
Archean crustal evolution in the northern Yilgam Craton:
U-Pb and Hf-isotope evidence from detrital zircons. Precambrian Research, 231-282.

Soderlund, U., Patchett, J., Vervoort, J. & Isachsen, C. 2004.
The Lu-176 decay constant determined by Lu-Hf and U-Pb isotope systematics of Precambrian mafic intrusions.
Earth And Planetary Science Letters 219(3-4), 311-324.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_hf}
\alias{plot_hf}
\title{Plot Lu-Hf data}
\usage{
plot_hf(
  dat,
  range = c(0, 4560),
  plot_type = "ehf",
  guide = TRUE,
  x_errors = FALSE,
  y_errors = FALSE,
  error_bars = FALSE,
  contours = FALSE,
  x_bandwidth = NULL,
  y_bandwidth = NULL,
  contour_data = NULL,
  combine_contours = FALSE,
  constants
)
}
\arguments{
\item{dat}{data.frame}

\item{range}{range to display}

\item{plot_type}{'ehf'=epsilon-Hf; 'hfhf'=176/177Hf}

\item{guide}{logical, show legend?}

\item{x_errors}{logical, include x errorbars}

\item{y_errors}{logical, include y errorbars}

\item{error_bars}{logical, include errorbars}

\item{contours}{logical, plot contours?}

\item{x_bandwidth}{bandwidth 2dkde x-direction}

\item{y_bandwidth}{bandwidth 2dkde y-direction}

\item{contour_data}{data.frame containing data to contour}

\item{combine_contours}{logical combine contouring data}

\item{constants}{vector of constants which must be in the order
decay constant 176Lu, 176/177Hf CHUR, 176Lu/177Hf CHUR, 176/177Hf DM and
176Lu/177Hf DM}
}
\description{
Plot Lu-Hf data as both epsilon-Hf vs. age and 176/177Hf vs. age.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{satkoski_2d_matrix}
\alias{satkoski_2d_matrix}
\title{Pairwise 2d Satkoski likeness}
\usage{
satkoski_2d_matrix(dat, bw = c(30, 2.5), digits = 3)
}
\arguments{
\item{dat}{data.frame}

\item{bw}{vector of density bandwidths}

\item{digits}{number, round result to significant digits}
}
\description{
Populate a matrix with pairwise Satkoski 12 likeness.
}
\references{
Satkoski, A.M., Wilkinson, B.H., Hietpas, J., Samson, S.D., 2013.
Likeness among detrital zircon populations - An approach to the comparison of
age frequency data in time and space. GSA Bulletin 125, 1783-1799.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_dens_hist}
\alias{plot_dens_hist}
\title{Plot 1d density with histogram}
\usage{
plot_dens_hist(
  dat,
  bw = 30,
  binwidth = 50,
  type = "kde",
  age_range = c(0, 4560),
  facet = FALSE,
  fixed_y = FALSE,
  step = 200
)
}
\arguments{
\item{dat}{data.frame}

\item{bw}{density bandwidth}

\item{binwidth}{histogram binwidth}

\item{type}{'kde': traditional KDE 'pdd': detrital zircon PDD}

\item{age_range}{range over which to calculate density}

\item{facet}{logical, facet samples?}

\item{fixed_y}{logical, fixed y-axis?}

\item{step}{specify x-axis steps}
}
\value{
ggplot2 1d density plot with histogram
}
\description{
Creates density and histogram plot of U-Pb data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_labels}
\alias{plot_labels}
\title{Labels for ggplot2 plots}
\usage{
plot_labels(xlab = "Age (Ma)", ylab = "Density")
}
\arguments{
\item{xlab}{X-axis label}

\item{ylab}{Y-axis label}
}
\value{
Returns ggplot2 labels
}
\description{
Convenience function to label ggplot2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_bw_theme}
\alias{plot_bw_theme}
\title{Stripped down theme for ggplot2}
\usage{
plot_bw_theme()
}
\value{
Returns ggplot2 theme
}
\description{
Stripped down theme for ggplot2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{make_tiling}
\alias{make_tiling}
\title{Produce data.frame of 1-O matrix suitable for geom_tile}
\usage{
make_tiling(dat, type)
}
\arguments{
\item{dat}{data.frame}

\item{type}{What to calculate}
}
\description{
Produce data.frame of 1-O matrix suitable for geom_tile
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_text_options}
\alias{plot_text_options}
\title{Modify text options of plots}
\usage{
plot_text_options(
  font_name = "Helvetica",
  title_size = 10,
  label_size = 7,
  legend_size = 10,
  strip_text_y_size = 8
)
}
\arguments{
\item{font_name}{Name of font to use}

\item{title_size}{Font size of x- and y-axis titles}

\item{label_size}{Font size of x- and y-axis tick labels}

\item{legend_size}{Font size of legend}

\item{strip_text_y_size}{Font size of vertical panel text}
}
\description{
Modify text options of plots
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{reimink}
\alias{reimink}
\title{Calculate upper and lower concordia intercepts from discordant detrital
zircon data}
\usage{
reimink(dat, step = 5)
}
\arguments{
\item{dat}{data.frame}

\item{step}{Chord spacing}
}
\description{
Calculate upper and lower concordia intercepts from discordant detrital
zircon data
}
\references{
Reimink, J.R., Davies, J.H.F.L., Waldron, J.W.F., Rojas, X.
(2016). Dealing with discordance: a novel approach for analysing U-Pb
detrital zircon datasets. Journal of the Geological Society.
doi: 10.1144/jgs2015-114
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{quant_bounds}
\alias{quant_bounds}
\title{Calculate confidence bands for lower and upper quartile}
\usage{
quant_bounds(dat, column = "t_dm2", alpha = 0.05)
}
\arguments{
\item{dat}{data.frame}

\item{column}{column to use for calculations}

\item{alpha}{alpha-level}
}
\description{
Calculate confidence bands for lower and upper quartile
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{combine_matrices}
\alias{combine_matrices}
\title{Combine two square matrices}
\usage{
combine_matrices(mat1, mat2)
}
\arguments{
\item{mat1}{Matrix for upper triangle}

\item{mat2}{Matrix for lower triangle}
}
\description{
Combine two square matrices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{find_plot_max}
\alias{find_plot_max}
\title{Find maximum value for plotting.}
\usage{
find_plot_max(x, accuracy = 100)
}
\arguments{
\item{x}{vector of values}

\item{accuracy}{round to nearest}
}
\description{
Find the maximum value for histogram plotting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_detzrcr.R
\name{run_detzrcr}
\alias{run_detzrcr}
\title{Launch shiny interface}
\usage{
run_detzrcr(...)
}
\arguments{
\item{...}{Pass arguments on to shiny::runApp}
}
\description{
Launch shiny interface
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\docType{data}
\name{lambda_lu}
\alias{lambda_lu}
\title{Decay constant of 176Lu.}
\format{
An object of class \code{numeric} of length 1.
}
\usage{
lambda_lu
}
\description{
Decay constant of 176Lu.
}
\references{
Soderlund, U., Jonathan Patchett, P., Vervoort, J.D. and Isachsen
C.E. 2004. The 176Lu decay constant determined by Lu^Hf and U^Pb isotope
systematics of Precambrian mafic intrusions. Earth and Planetary Science
Letters 219, 311-324.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{concX}
\alias{concX}
\title{Calculate U235 at given age}
\usage{
concX(age)
}
\arguments{
\item{age}{input age}
}
\description{
Calculate U235 at given age
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{hf_lines}
\alias{hf_lines}
\title{Produce CHUR and DM lines}
\usage{
hf_lines(range = c(0, 4560), plot_type = "ehf", constants)
}
\arguments{
\item{range}{range over which to calculate lines}

\item{plot_type}{'ehf' = epsilon-Hf; any thing else gives 176/177Hf}

\item{constants}{vector of constants which must be in the order
decay constant 176Lu, 176/177Hf CHUR, 176Lu/177Hf CHUR, 176/177Hf DM and
176Lu/177Hf DM}
}
\description{
Calculate CHUR and DM lines used for epsilon-Hf vs. age and 176/177Hf vs. age
plots.
}
\references{
Griffin, W., Pearson, N., Belousova, E., Jackson, S., van
Achterbergh, E., O'Reilly, S. and Shee, S. 2000. The Hf isotope composition
of cratonic mantle:
LAM-MC-ICPMS analysis of zircon megacrysts in kimberlites.
Geochimica et Cosmochimica Acta 64(1), 133-147.

Soderlund, U., Jonathan Patchett, P., Vervoort, J.D. and Isachsen
C.E. 2004. The 176Lu decay constant determined by Lu-Hf and U-Pb isotope
systematics of Precambrian mafic intrusions. Earth and Planetary Science
Letters 219, 311-324.

Bouvier, A., Vervoort, J.D. and Jonathan Patchett P. 2008. The
Lu-Hf and Sm-Nd isotopic composition of CHUR: Constraints from unequilibrated
chondrites and implications for the bulk composition of terrestrial planets.
Earth and Planetary Science Letters 273, 48-57.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{calc_dens_hist}
\alias{calc_dens_hist}
\title{Calculate scaled 1d density}
\usage{
calc_dens_hist(
  dat,
  binwidth = 50,
  bw = 30,
  type = "kde",
  age_range = c(0, 4560)
)
}
\arguments{
\item{dat}{data.frame}

\item{binwidth}{Histogram binwidth}

\item{bw}{Density bandwidth}

\item{type}{'kde': KDE; 'pdd': detrital zircon PDD}

\item{age_range}{Age range to calculated density over}
}
\value{
Returns density
}
\description{
Calculates 1d density of age data and scales it so that it can be plotted
in the same plot of a histogram of the age data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\docType{data}
\name{hfhf_dm}
\alias{hfhf_dm}
\title{176Lu/177Hf value of DM.}
\format{
An object of class \code{numeric} of length 1.
}
\usage{
hfhf_dm
}
\description{
176Lu/177Hf value of DM.
}
\references{
Griffin, W., Pearson, N., Belousova, E., Jackson, S., van
Achterbergh, E., O'Reilly, S. and Shee, S. 2000. The Hf isotope composition
of cratonic mantle:
LAM-MC-ICPMS analysis of zircon megacrysts in kimberlites.
Geochimica et Cosmochimica Acta 64(1), 133-147.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{populate_matrix}
\alias{populate_matrix}
\title{Populate matrix}
\usage{
populate_matrix(dat, FUN, ...)
}
\arguments{
\item{dat}{data.frame}

\item{FUN}{Function used to populate matrix}

\item{...}{Additional parameters passed to function}
}
\value{
Populated matrix
}
\description{
Populate matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_axis_lim}
\alias{plot_axis_lim}
\title{Axes limits for ggplot2}
\usage{
plot_axis_lim(xlim = c(0, 4560), step = 200, ylim = NULL)
}
\arguments{
\item{xlim}{x-axis limit}

\item{step}{specify x-axis steps}

\item{ylim}{y-axis limit}
}
\value{
list of ggplot2::coord_cartesian object
}
\description{
Convenience function to change axes limits for ggplot2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\docType{data}
\name{luhf_zrc}
\alias{luhf_zrc}
\title{176Lu/177Hf value of average continental crust.}
\format{
An object of class \code{numeric} of length 1.
}
\usage{
luhf_zrc
}
\description{
176Lu/177Hf value of average continental crust.
}
\references{
Griffin, W., Belousova, E., Shee, S., Pearson, N. and O'Reilly,
S. 2004. Archean crustal evolution in the northern Yilgam Craton:
U-Pb and Hf-isotope evidence from detrital zircons. Precambrian Research,
231-282.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{calc_dens}
\alias{calc_dens}
\title{Calculate 1D density of age data}
\usage{
calc_dens(dat, bw = 30, type = "kde", age_range = c(0, 4560))
}
\arguments{
\item{dat}{data.frame containing at least ages and percentage of discordancy}

\item{bw}{Bandwidth}

\item{type}{Type to calculate 'kde': proper KDE; 'pdd': detrital zircon PDD}

\item{age_range}{Range over which to calculate density}
}
\value{
Density
}
\description{
Calculate the 1d density of U-Pb age data using KDE or PDD.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{calc_dkw}
\alias{calc_dkw}
\title{Dvoretzky-Kiefer-Wolfowitz inequality}
\usage{
calc_dkw(dat, column = "age", alpha = 0.05)
}
\arguments{
\item{dat}{data.frame}

\item{column}{which column to use}

\item{alpha}{Desired alpha level}
}
\value{
data.frame with ecdf and confidence bands
}
\description{
Calculate confidence bands for ecdfs using the Dvoretzky-Kiefer-Wolfowitz
inequality.
}
\references{
Dvoretzky, A., Kiefer, J., Wolfowitz, J., 1956. Asymptotic
Minimax Character of the Sample Distribution Function and of the Classical
Multinomial Estimator. Ann. Math. Stat. 27, 642-669.
doi:10.1214/aoms/1177728174
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{tiling}
\alias{tiling}
\title{Apply tile_func to vector}
\usage{
tiling(z)
}
\arguments{
\item{z}{1-O parameter vector}
}
\description{
Apply tile_func to vector
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{check_conc}
\alias{check_conc}
\title{Check concordancy of input ages}
\usage{
check_conc(dat, disc_lim = 10)
}
\arguments{
\item{dat}{data.frame containing at least ages and percentage of discordancy}

\item{disc_lim}{Discordancy limit}
}
\value{
Concordant data
}
\description{
Check the concordancy of the U-Pb data and return the data within the desired
discordancy limit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{satkoski_2d}
\alias{satkoski_2d}
\title{Calculate 2d (age and Lu-Hf) likeness of detrital zircon populations}
\usage{
satkoski_2d(x, y, bw = c(30, 2.5), digits = 3)
}
\arguments{
\item{x}{vector}

\item{y}{vector}

\item{bw}{vector of density bandwidths}

\item{digits}{number, round result to significant digits}
}
\description{
Calculates the likeness of detrital zircon populations in 2 dimensions after
Satoski et al. (2013).
}
\references{
Satkoski, A.M., Wilkinson, B.H., Hietpas, J., Samson, S.D., 2013.
Likeness among detrital zircon populations - An approach to the comparison of
age frequency data in time and space. GSA Bulletin 125, 1783-1799.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{dzr_mix}
\alias{dzr_mix}
\title{Calculate mixing model}
\usage{
dzr_mix(mu1, sig1, mu2, sig2)
}
\arguments{
\item{mu1}{first mean}

\item{sig1}{first standard deviation}

\item{mu2}{second mean}

\item{sig2}{second standard deviation}
}
\description{
Gaussian mixing model for detrital zircon data, using lower quantile upper
quantile plot
}
\examples{
dzr_mix(500, 50, 1000, 100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{find_maxima}
\alias{find_maxima}
\title{Find maxima.}
\usage{
find_maxima(dist, xmin, inc)
}
\arguments{
\item{dist}{distribution.}

\item{xmin}{minimum value of distribution.}

\item{inc}{increment.}
}
\description{
Find maxima.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_quantiles}
\alias{plot_quantiles}
\title{Plot quantiles}
\usage{
plot_quantiles(
  dat,
  column = "t_dm2",
  conf = FALSE,
  alpha = 0.05,
  type = 8,
  guide = TRUE,
  mix = FALSE,
  mix_data = NULL
)
}
\arguments{
\item{dat}{data.frame}

\item{column}{which column in data.frame to use}

\item{conf}{logical, plot confidence interval}

\item{alpha}{alpha-level}

\item{type}{type of quantile calculation (passed on to stats::quantile)}

\item{guide}{logical, show legend?}

\item{mix}{logical, add mixing model}

\item{mix_data}{mixing model data}
}
\description{
Plot quantiles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{find_plot_min_max}
\alias{find_plot_min_max}
\title{Wrapper function for \code{find_plot_min} and \code{find_plot_max}}
\usage{
find_plot_min_max(x, accuracy = 100)
}
\arguments{
\item{x}{Age data}

\item{accuracy}{Round to nearest}
}
\value{
Returns vector of minimum and maximum plotting values
}
\description{
Find the minimum and maximum values for histogram plotting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{Natal_group}
\alias{Natal_group}
\title{Dataset Natal group}
\description{
Detrital zircon U-Pb age and Lu-Hf data from the Natal Group, KwaZulu-Natal,
South-Africa (Kristoffersen et al. 2016).
}
\details{
Used as example of proper format of a csv-file for input in the shiny
interface.
}
\references{
Kristoffersen, M., Andersen, T., Elburg, M.A., Watkeys, M.K.,
2016. Detrital zircon in a supercontinental setting: locally derived and
far-transported components in the Ordovician Natal Group, South Africa.
J. Geol. Soc. London. 173, 203-215. doi:10.1144/jgs2015-012
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_reimink}
\alias{plot_reimink}
\title{Plot likelihood of intercept ages}
\usage{
plot_reimink(dat)
}
\arguments{
\item{dat}{data.frame}
}
\description{
Plot likelihood of intercept ages
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{o_param_matrix_age}
\alias{o_param_matrix_age}
\title{Populate matrix with age 1-O}
\usage{
o_param_matrix_age(dat, alpha = 0.05, digits = 2)
}
\arguments{
\item{dat}{data.frame}

\item{alpha}{alpha level}

\item{digits}{number of digits}
}
\value{
matrix of 1-O for ages
}
\description{
Populate matrix with age 1-O
}
\references{
Andersen, T., Elburg, M., Cawthorn-Blazeby, A., 2015.
U-Pb and LuHf zircon data in young sediments reflect sedimentary recycling
in eastern South Africa. J. Geol. Soc. London. 2006-2015.
doi:10.1144/jgs2015-006
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\docType{data}
\name{lambda_u235}
\alias{lambda_u235}
\title{Decay constants of 235U.}
\format{
An object of class \code{numeric} of length 1.
}
\usage{
lambda_u235
}
\description{
Decay constants of 235U.
}
\references{
Steiger, R. & Jager, E. 1977. SUBCOMMISSION ON GEOCHRONOLOGY -
CONVENTION ON USE OF DECAY CONSTANTS IN GEOCHRONOLOGY AND COSMOCHRONOLOGY.
Earth And Planetary Science Letters 36(3), 359-362.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{calc_quantiles}
\alias{calc_quantiles}
\title{Calculate quantiles}
\usage{
calc_quantiles(dat, column = "t_dm2", alpha = 0.05, type = 8)
}
\arguments{
\item{dat}{data.frame}

\item{column}{which column in data.frame to use}

\item{alpha}{alpha-level (not yet used)}

\item{type}{type of quantile calculation (passed on to stats::quantile)}
}
\description{
Split up data.frame by sample-column and calculate quantiles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_tile}
\alias{plot_tile}
\title{Tile plot of 1-O matrix}
\usage{
plot_tile(dat, type)
}
\arguments{
\item{dat}{data.frame}

\item{type}{What to plot}
}
\description{
Tile plot of 1-O matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_ecdf}
\alias{plot_ecdf}
\title{Plot ecdf}
\usage{
plot_ecdf(
  dat,
  mult_ecdf = FALSE,
  column = "age",
  conf = FALSE,
  guide = TRUE,
  alpha = 0.05
)
}
\arguments{
\item{dat}{data.frame}

\item{mult_ecdf}{logical, plot several ecdfs in same plot}

\item{column}{which column to use}

\item{conf}{logical, plot confidence bands}

\item{guide}{logical, show legend}

\item{alpha}{alpha-level used for confidence bands}
}
\description{
Plot ecdf for U-Pb age or Lu-Hf model age data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_point_scale}
\alias{plot_point_scale}
\title{Add manual shape scale to scatter plot}
\usage{
plot_point_scale()
}
\description{
Add manual shape scale to scatter plot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\docType{data}
\name{luhf_chur}
\alias{luhf_chur}
\title{176Lu/177Hf value of CHUR.}
\format{
An object of class \code{numeric} of length 1.
}
\usage{
luhf_chur
}
\description{
176Lu/177Hf value of CHUR.
}
\references{
Bouvier, A., Vervoort, J.D. and Jonathan Patchett P. 2008. The
Lu-Hf and Sm-Nd isotopic composition of CHUR: Constraints from unequilibrated
chondrites and implications for the bulk composition of terrestrial planets.
Earth and Planetary Science Letters 273, 48-57.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculations.R
\name{calc_o_param}
\alias{calc_o_param}
\title{Calculate 1-O}
\usage{
calc_o_param(dat1, dat2, column, alpha = 0.05, digits = 2)
}
\arguments{
\item{dat1}{data.frame}

\item{dat2}{data.frame}

\item{column}{string of name of column to use ('age' or 't_dm2')}

\item{alpha}{alpha level}

\item{digits}{number of digits}
}
\value{
1-O
}
\description{
Calculate 1-O
}
\references{
Andersen, T., Elburg, M., Cawthorn-Blazeby, A., 2015.
U-Pb and Lu-Hf zircon data in young sediments reflect sedimentary recycling
in eastern South Africa. J. Geol. Soc. London. 2006-2015.
doi:10.1144/jgs2015-006
}
